/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport;

import java.util.*;
import net.sf.bloof.scm.cvsplugin.*;
import net.sf.bloof.db.*;
import net.sf.bloof.*;


/**
 * The Config class parses the user specified config file
 * and provides methods for accessing the user defined properties.
 * The following CVS properties must be configured in your
 * properties file:
 * <ul>
 * <li>cvsroot: the path to the CVS repository
 * <li>cvsmodule: the module name of the CVS project
 * </ul>
 *
 * By default, we store the database in your home directory
 * under a subdirectory called "bloof". If you would like to
 * create a database under a different location, then the following
 * configuration parameters will allow you to do so:
 * <ul>
 * <li>dburl: the path to the database file
 * <li>dbuser: the user to access the database
 * <li>dbpassword: the password to access the database
 * </ul>
 *
 * It is not recommended to use external CVS servers because
 * doing so will be slow and will use up a lot of your network
 * resources.
 *
 * @author David James
 */
public class Config extends Properties {


    /**
     * Read the location of a CVS repository
     * @see net.sf.bloof.script.CommandParser
     */
    public CvsAccess cvs() throws NoSuchFieldException,
            InvalidRepositoryLocationException {

        // Get repository location
        String cvsRoot = getRequiredProperty("cvsroot");
        String cvsModule = getRequiredProperty("cvsmodule");
        RepositoryLocation rLoc = new RepositoryLocation(cvsRoot);

        // Get login details
        LoginDetails lDetail = new LoginDetails(rLoc.getUserName(), null);

        // Create new CvsAccess object using repository location
        // and login details
        return new CvsAccess(rLoc, lDetail, cvsModule, "cvs access");
    }


    /**
     * Find out where our SQL database is located
     * @see net.sf.bloof.script.CommandParser
     */
    public DbAccess db() throws NoSuchFieldException {

        // If the database URL is not specified, then
        // use the internal Bloof database
        String dbUrl = getProperty("dburl");
        if (dbUrl == null) {
            return Bloof.INTERNAL_DATABASE;
        } else {
            // Otherwise, create and return a new DbAccess object
            // which describes the location, username and password
            // required to access the database
            String dbUser = getRequiredProperty("dbuser");
            String dbPassword = getRequiredProperty("dbpassword");
            return new DefaultDbAccess("database", dbUser, dbPassword, dbUrl);
        }
    }

    /** Get the names of the reports
     * @throws NoSuchFieldException when no graphs are specified
     * @return List of reports */
    public String[] getGraphNames() throws NoSuchFieldException {
        String graphNames = getRequiredProperty("graphs");
        return graphNames.split(",");
    }


    /** Get a required property
     * @param name the name of the property
     * @return the value of the property
     * @throws NoSuchFieldException if this field does not exist
     */
    String getRequiredProperty(final String name) throws NoSuchFieldException {
        String value = getProperty(name);
        if (name == null) {
            throw new NoSuchFieldException(name);
        }
        return value;
    }
}
