/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;

import ca.utoronto.JCVSReport.cvsPlugin.Cvs;
import ca.utoronto.JCVSReport.report.Report;

import net.sf.bloof.db.DbAccess;
import net.sf.bloof.db.McKoiControl;
import net.sf.bloof.scm.cvsplugin.CvsAccess;

/**
 * Represents a project with an associated database
 * and CVS repository. Enables user to import, update and close
 * the project, as well as to produce a report.
 *
 * @author Polina Gohshtein
 */
public class JCVSProject {

    private MetricDatabase mDatabase = null;
    private Report mReport;
    private CvsAccess mCvsAccess;
    private DbAccess mDbAccess;
    private String[] mGraphs;

    /** Creates JCVSReport given database access and SCM access information.
     * @param aCvsAccess CVS access information
     * @param aDbAccess Database access information
     * @param aGraphs The graphs to be generated
     */
    public JCVSProject(final CvsAccess aCvsAccess,
        final DbAccess aDbAccess, final String[] aGraphs) {
        mCvsAccess = aCvsAccess;
        mDbAccess = aDbAccess;
        mGraphs = aGraphs;
    }

    /** Imports a new project by creating a new database for it,
     * and importing the CVS data for the whole lifetime of the
     * project.
     * @throws Exception
     */
    public void importProject() throws Exception {
        try {
            /* Create a connection to the database */
            Connection connection = McKoiControl.create(mDbAccess);
            McKoiControl.createIndices();
            mDatabase = new MetricDatabase(connection);
        } catch (IOException e) {
            throw new SQLException("Could not open McKoi database: "
                    + e.toString());
        }

        /* Load graph objects */
        mReport = new Report(mDatabase);
        mReport.addGraphs(mGraphs);

        /* Create a cvs object */
        Cvs cvs = new Cvs(mCvsAccess);

        /* Import statistics from cvs to the database */
        UpdateStatistics updateStats = new UpdateStatistics(mDatabase, cvs);
        updateStats.importStats(mReport);
    }

    /** Updates the project since last time it has been updated.
     * @throws Exception
     */
    public void updateProject() throws Exception {

        try {
            /* Connect to an existing database */
            Connection connection = McKoiControl.open(mDbAccess);
            McKoiControl.createIndices();
            mDatabase = new MetricDatabase(connection);

        } catch (IOException e) {
            throw new SQLException("Could not open McKoi database: "
                    + e.toString());
        }

        /* Load graph objects */
        mReport = new Report(mDatabase);
        mReport.addGraphs(mGraphs);

        /* Create a cvs object */
        Cvs cvs = new Cvs(mCvsAccess);

        /* Update the database with statistics from cvs */
        UpdateStatistics updateStats = new UpdateStatistics(mDatabase, cvs);
        updateStats.updateStats(mReport);
    }

    /** Produces an HTML report with the specified file name and
     * in the specified directory.
     * @param file The file name
     * @param directory The directory
     * @throws Exception
     */
    public void produceReport(final String file,
        final String directory) throws Exception {

        mReport.write(file, directory);
        /* Display message */
        System.out.println("Wrote " + file);
    }

    /** Close the database.
     * @throws SQLException if the database could not be found or started
     * @throws IOException if the database could not be found or started
     */
    public void closeProject() throws IOException, SQLException {
        if (mDatabase != null) {
            mDatabase.close();
            mDatabase = null;
        }
    }

    /** Access the internal report object */
    public Report getReport() {
        return mReport;
    }
}
