/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport;

import net.sf.bloof.*;
import net.sf.bloof.db.*;

import java.io.*;
import java.sql.*;
import java.util.Iterator;

import ca.utoronto.JCVSReport.cvsPlugin.History;


/**
 * A McKoi database for storing metrics based on statistics
 * gathered from CVS repositories
 * @author David James
 * @author Polina Gohshtein
 */
public class MetricDatabase extends Database {

    // SQL query for getting a file with a given path
    private static final String SQL_GET_DELETEDFILE =
        "SELECT pathname FROM File, DeletedFile "
            + "WHERE File.pathname=DeletedFile.file "
            + "AND File.pathname=?",
        SQL_GET_REVISIONID =
            "SELECT revision_id FROM File, Revision WHERE "
            + "File.pathname=Revision.file "
            + "AND File.pathname=? AND Revision.version=?",
        SQL_ADD_CVSOPERATION = "INSERT INTO CVSOperation "
            + "( operation, revision_id, tstamp, developer) "
            + "VALUES ( ?,?,?,?)";

    private PreparedStatement getRevisionIDStatement;
    private PreparedStatement getDeletedFileStatement;
    private PreparedStatement addCVSOperationStatement;

    /** Initialize from an existing connection
     * @param connection An existing connection */
    public MetricDatabase(Connection connection) throws SQLException {

        // Initialize connection
        super(connection);
        mConnection = connection;

        // Prepare SQL statements
        getRevisionIDStatement =
            mConnection.prepareStatement(SQL_GET_REVISIONID);
        getDeletedFileStatement =
            mConnection.prepareStatement(SQL_GET_DELETEDFILE);
        addCVSOperationStatement =
            mConnection.prepareStatement(SQL_ADD_CVSOPERATION);

        /* Define an exception handler */
        Bloof.addBloofExceptionListener(new BloofExceptionListener() {
            public void bloofExceptionHappend(BloofException exception) {
                exception.printStackTrace();
                System.out.flush();
                System.err.flush();
                System.exit(1);
            }
        });
    }

    /** Close the metric database */
    public void close() throws IOException, SQLException {
        super.close();
    }


    /** Prepare an SQL statement for execution
     * @return The prepared statement
     * @param statement An SQL statement
     * @throws SQLException if the statement cannot be compiled
     */
    public PreparedStatement prepareStatement(String statement)
            throws SQLException {
        return mConnection.prepareStatement(statement);
    }

    /** Populate the database with CVS history data.
     * @param historyIter An iterator over CVS history records
     * @throws SQLException
     */
    public void populateWithHistory(Iterator historyIter)
        throws SQLException {

        // Turn off AutoCommit so that we can commit a series
        // of operations as a single transaction (it's faster)
        mConnection.setAutoCommit(false);

        // Keep the list of developers in memory
        // in order to redice the number of database accesses
        fillDeveloperHash();

        // Iterate over the history records and add each
        // record to the database
        while (historyIter.hasNext()) {
            History record = (History) historyIter.next();
            addCvsOperation(record);
        }

        // Turn AutoCommit back on and commit the transaction
        mConnection.setAutoCommit(true);
    }

    /** Query the revision id of the file given the file
     * pathname and version.
     * @param file Pathname of the file in CVS.
     * @param version Version of the file in CVS.
     * @return revisionId
     * @throws SQLException
     */
    public String getRevisionId(String file, String version)
        throws SQLException {

        // Get the revision ID from the database
        getRevisionIDStatement.setString(1, file);
        getRevisionIDStatement.setString(2, version);
        ResultSet result = getRevisionIDStatement.executeQuery();

        // Return the revision ID
        String revisionId = null;
        if (result.next()) {
            revisionId = result.getString(1);
        }
        return revisionId;
    }

    /** Checks if the file with the given path has been deleted
     * from CVS.
     * @param filepath The path to the file
     * @return True if this file has been deleted from CVS, i.e. if
     * it exists in the 'DeletedFile' table in our database.
     * @throws SQLException
     */
    public boolean isDeletedFile(String filepath) throws SQLException {

        // Run a select query to get a deleted file with the
        // given file path
        getDeletedFileStatement.setString(1, filepath);
        ResultSet result = getDeletedFileStatement.executeQuery();
        // If the result set is not empty, it means that there exists
        // a deleted file with this file path in the database
        if (result.next()) {
            return true;
        }
        return false;
    }

    /** Sets auto commit to the given value.
     * @param value Boolean value to set autocommit to.
     * @throws SQLException
     */
    public void setAutoCommit(boolean value) throws SQLException {
        mConnection.setAutoCommit(value);
    }

    /** Update the database by adding the cvs operation
     * captured by the given history record.
     * @param aRecord The history record.
     * @throws SQLException
     */
    protected void addCvsOperation(History aRecord) throws SQLException {

        // Check if a specified developer already exists in the database.
        // If it does not - add it.
        if (!mDeveloperNames.containsKey(aRecord.getUser())) {
            addNewDeveloper(aRecord.getUser());
            mDeveloperNames.put(aRecord.getUser(), aRecord.getUser());
        }

        // Query the database to find out if it has a record of this
        // file in either the 'File' table or the 'DeletedFile' table.
        String revisionId = getRevisionId(aRecord.getQualifiedFilename(),
            aRecord.getRevision());

        // If there is no file with such revision id, check the deleted
        // files.
        if (revisionId == null) {
            // If this file does not exist in the deleted files,
            // then do not record the operation.
            if (!isDeletedFile(aRecord.getQualifiedFilename())) {
                return;
            }
        }

        // Add the CVS operation to the database
        addCVSOperationStatement.setString(1, aRecord.getOperation());
        addCVSOperationStatement.setString(2, revisionId);
        addCVSOperationStatement.setTimestamp(3,
                aRecord.getTimestamp());
        addCVSOperationStatement.setString(4, aRecord.getUser());
        addCVSOperationStatement.executeUpdate();
    }
}
