/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport;

import java.io.*;

import net.sf.bloof.scm.ScmAccessException;
import java.util.*;

import org.apache.commons.cli.*;


/**
 * <p>The Presentation component is the entry point of JCVSReport, i.e. this
 * component is the main class that will be called when the user invokes
 * JCVSReport via command line.</p>
 *
 * <p>The Presentation class expects the name of a .properties file to be passed
 * in as a command-line argument. Using this information, the Presentation class
 * performs the following functions:</p>
 *
 * <ol>
 * <li> Call the 'Config' class to parse the .properties file and get the CVS
 *      and database information specified by the user.
 * <li> Ask the 'MetricDatabase' class to bring the specified database up to
 *      date with the specified CVS repository
 * <li> Ask the 'Report' class to display all of the graphs specified in
 *      the .properties file
 * </ol>
 *
 * @author David James
 * @author Polina Gohshtein
 */
public final class Presentation {

    private static final String OP_UPDATE = "u";
    private static final String OP_CONFIG = "c";
    private static final String OP_TEST = "t";

    /** command line options */
    private static final Options CLOPTS = new Options();
    static {
        // Create Options object
        Options options = new Options();
        // Add the boolean -update option
        Option update = new Option(OP_UPDATE, "update the database");
        // Add the boolean -test option
        Option test = new Option(OP_TEST, "run test suite");

        // Add the config file option
        OptionBuilder.withArgName("file");
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("use given file for configuration");
        Option configfile = OptionBuilder.create(OP_CONFIG);
        CLOPTS.addOption(configfile);
        CLOPTS.addOption(update);
        CLOPTS.addOption(test);
    }

    /** Hide the default constructor */
    private Presentation() { }

    /** Main subroutine
     * @see Presentation for a detailed description */
    public static void main(final String[] args) {
        // Read properties from command-line
        CommandLineParser parser = new PosixParser();
        try {
            // Parse the command line arguments
            CommandLine line = parser.parse(CLOPTS, args);

            // Run the test suite, if they asked for it
            if (line.hasOption(OP_TEST)) {
                AllTests.main();
                return;
            }

            // Validate that config file has been set
            if (!line.hasOption(OP_CONFIG)) {
                System.out.println("Must specify a config file.");
                usage();
            }

            // Load config file from disk
            Config config = new Config();
            config.load(new FileInputStream(line.getOptionValue("c")));

            // Create a JCVS project for the specified CVS, database
            // and with the specified graphs
            JCVSProject project = new JCVSProject(config.cvs(), config.db(),
                    config.getGraphNames());

            // If the update option is specified, then update the
            // project. Otherwise, import it.
            if (line.hasOption(OP_UPDATE)) {
                project.updateProject();
            } else {
                project.importProject();
            }

            // Produce a report
            project.produceReport("report.html", ".");

            // Close the project
            project.closeProject();

        } catch (ScmAccessException e) {
            System.out.println(e.toString()
                + "\nPlease check cvs repository location "
                + "and cvsmodule in the configuration file.");
        } catch (MissingResourceException e) {
            System.out.println(e.toString()
                + "\nInvalid graph. Please check 'graph' property "
                + "in the configuration file.");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /** Prints the usage information. **/
    private static void usage() {
        final HelpFormatter hf = new HelpFormatter();
        hf.printHelp("JCVSReport", CLOPTS);
        System.exit(1);
    }
}