/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport;

import ca.utoronto.JCVSReport.cvsPlugin.Cvs;
import ca.utoronto.JCVSReport.report.Report;

import net.sf.bloof.metrics.TimeInterval;

import java.sql.ResultSet;
import java.sql.Timestamp;
import java.sql.PreparedStatement;
import java.util.Date;

/** UpdateStatistics brings the statistics in the database up to
 * date with the CVS repository.
 *
 * @author David James
 * @author Polina Gohshtein
 */
public class UpdateStatistics {
    private Cvs mCvs;
    private MetricDatabase mDatabase;

    /** Creates UpdateStatistics given a database and a cvs.
     * @param aDatabase The database to be updated
     * @param aCvs The Cvs to be used for information retrieval
     */
    public UpdateStatistics(final MetricDatabase aDatabase, final Cvs aCvs) {
        mCvs = aCvs;
        mDatabase = aDatabase;
    }

    /** Updates the database with cvs metadata starting at a particular date.
     * @param fromDate the date at which we should start the update
     * @throws Exception
     */
    public void updateCvsMetadata(Date fromDate) throws Exception {
        // Update database with data from the cvs log
        mDatabase.populateDatabase(mCvs.getCvsLogData(fromDate));

        // Update database with data from the cvs history
        mDatabase.populateWithHistory(mCvs.getCvsHistoryData(fromDate));
    }

    /** Import the cvs metadata to the database
     * @throws Exception
     */
    public void importCvsMetadata() throws Exception {
        // Import database with data from the cvs log
        mDatabase.populateDatabase(mCvs.getCvsLogData());

        // Import database with data from the cvs history
        mDatabase.populateWithHistory(mCvs.getCvsHistoryData());
    }

    /** Update all of ring the statistics in the database up to date with the
     * CVS repository.
     *
     * 1) Ask Bloof to update the database with the CVS metadata.
     * 2) For each new revision, read the file into memory and gather
     *    the metrics that are needed for our current report
     *
     * @param report The report object with information about metrics
     * and graphs to be imported
     * @throws Exception
     */
    public void updateStats(final Report report) throws Exception {
        // Get the last time the database was updated
        TimeInterval wholeTime = mDatabase.getTimespanBoundaries();
        Date date = wholeTime.getTo();

        // Update statistics
        updateCvsMetadata(date);
        updateJavaFiles(report, date);
    }

    /** Import the statistics
     * @param report The report object with information about metrics
     * and graphs to be updated
     * @throws Exception
     */
    public void importStats(final Report report) throws Exception {
        importCvsMetadata();
        importJavaFiles(report);
    }

    /** Update the Java files from disk
    *
    * @param report The report we're trying to generate
    * @param date The date at which we should start the update
    */
    private void updateJavaFiles(final Report report, Date date)
        throws Exception {
        /* Build query for selecting all revisions */
        PreparedStatement statement = mDatabase.prepareStatement(
                "SELECT revision_id, file, version FROM Revision "
              + "WHERE tstamp > ?");
        statement.setTimestamp(1, new Timestamp(date.getTime()));

        // Process the revisions
        processRevisions(report, statement.executeQuery());
    }

    /** Import all of the Java files from the repository
     * @param report The reports we want to generate */
    private void importJavaFiles(final Report report) throws Exception {
        processRevisions(report,
            mDatabase.executeQuery(
                "SELECT revision_id, file, version FROM Revision"));
    }

    /** Process the revisions returned by an SQL query and add them to the
     * database
     *
     * @param report The reports we want to generate
     * @param result The revisions returned by an SQL query
     * @throws Exception
     */
    private void processRevisions(final Report report, final ResultSet result)
        throws Exception {
        /* For speed, group all of our metric insertions as a single
         * transaction */
        mDatabase.setAutoCommit(false);

        /* For each new revision, read the file into memory and gather
         * appropriate metrics */
        while (result.next()) {
            String file = result.getString("file");

            // Only look at .java files for now
            if (file.endsWith(".java")) {
                String contents = mCvs.getCvsFile(result.getString("file"),
                        result.getString("version"));
                String revisionId = result.getString("revision_id");
                report.processRevision(revisionId, contents);
            }
        }

        /* Commit our transaction */
        mDatabase.setAutoCommit(true);
    }
}
