/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.cvsPlugin;

import java.io.IOException;
import java.text.SimpleDateFormat;

import net.sf.bloof.scm.ScmAccessException;
import net.sf.bloof.scm.ScmRevisionIterator;
import net.sf.bloof.scm.cvsplugin.CvsAccess;
import net.sf.bloof.scm.cvsplugin.RevisionIterator;
import ca.utoronto.SimpleProcess.*;

import java.util.Date;
import java.util.Iterator;

/** Cvs class acts as an interface to the CVS. It
 * contains operations to retrieve files from CVS
 * and CVS metadata.
 * @author Polina Gohshtein
 */
public class Cvs {

    private CvsAccess mAccessInformation;
    private SimpleDateFormat mDateFormat
        = new SimpleDateFormat("yyyy-MM-dd HH:mm ZZZZ");

    /** Creates a Cvs object with the given CVS access information.
     * @param accessInformation CVS access information
     */
    public Cvs(final CvsAccess accessInformation) {
        mAccessInformation = accessInformation;
    }

    /** Get a specific version of a specific file from a CVS repository
     * @param file The file we want to get
     * @param version The version of the file we want to get
     * */
    public String getCvsFile(final String file,
            final String version) throws IOException, InterruptedException {
        // Find the path of the file relative to the repository root
        String rootDir = mAccessInformation.getRepositoryRoot() + "/";
        String relativePath = file.replaceFirst(rootDir, "");

        // The cheapest and easiest way to get a file from a CVS repository is
        // just to call out to the command line client. So we do that.
        String[] command = {"cvs", "-d", mAccessInformation.getCvsRoot(),
                "checkout", "-r", version, "-p", relativePath };

        // Return the contents of the file
        return SimpleProcess.run(command);
    }

    /** Get project data from the CVS log for the whole lifetime of the project.
     * @return RevisionIterator object - an iterator over Cvs revisions
     * @throws IOException
     * @throws InterruptedException
     * @throws ScmAccessException
     */
    public ScmRevisionIterator getCvsLogData() throws IOException,
            InterruptedException, ScmAccessException {
        // Launch CVS to get the log file for this module
        String[] command = {"cvs", "-d" + mAccessInformation.getCvsRoot(),
                "rlog", mAccessInformation.getModuleName() };
        return getLogData(command);
    }

    /** Get project data from the CVS log from the specified date.
     * @param aFromDate The start date
     * @return RevisionIterator object - an iterator over Cvs revisions
     * @throws IOException
     * @throws InterruptedException
     * @throws ScmAccessException
     */
    public ScmRevisionIterator getCvsLogData(final Date aFromDate)
            throws IOException, InterruptedException,
            ScmAccessException {
        // Launch CVS to get the log file for this module
        String[] command = {"cvs", "-d" + mAccessInformation.getCvsRoot(),
                "rlog", "-d", ">" + mDateFormat.format(aFromDate),
                mAccessInformation.getModuleName()};
        return getLogData(command);
    }

    /** Get project data from the CVS history for the whole lifetime
     * of the project.
     * @return Iterator over the history records
     * @throws IOException
     * @throws InterruptedException
     * @throws ScmAccessException
     */
    public Iterator getCvsHistoryData() throws IOException,
            InterruptedException, ScmAccessException {
        String[] hisCommand = {"cvs", "-d" + mAccessInformation.getCvsRoot(),
                "history", "-a", "-e"};
        return getHistoryData(hisCommand);
    }

    /** Get project data from the CVS history from the specified date.
     * @param aFromDate The start date
     * @return Iterator over the history records
     * @throws IOException
     * @throws ScmAccessException
     */
    public Iterator getCvsHistoryData(final Date aFromDate)
            throws IOException, InterruptedException,
            ScmAccessException {
        String[] hisCommand = {"cvs", "-d" + mAccessInformation.getCvsRoot(),
                "history", "-D", mDateFormat.format(aFromDate), "-a", "-e"};
        return getHistoryData(hisCommand);
    }
    /** Runs a history command on CVS.
     * @param command The command used to get cvs history
     * @return an iterator over History records.
     * @throws IOException
     * @throws InterruptedException
     * @throws ScmAccessException
     */
    private Iterator getHistoryData(final String[] command) throws IOException,
            InterruptedException, ScmAccessException {

        SimpleProcess proc = new SimpleProcess(command);

        // Gobble up stderr to prevent deadlock on Win32
        new StreamGobbler(proc.stderr()).start();

        // Parse the output of 'cvs history'
        HistoryParser parser = new HistoryParser(proc.stdout(),
                mAccessInformation.getCvsRoot());
        parser.run();
        Iterator historyIter = parser.getHistory().iterator();

        // Check for appropriate exit code and throw exception if necessary
        if (proc.waitFor() != 0) {
            throw new ScmAccessException("Could not access SCM system.");
        }

        return historyIter;
    }

    /** Runs a log command on CVS.
     * @param command The command used to get cvs history
     * @return an Iterator over the revisions
     * @throws IOException
     * @throws InterruptedException
     * @throws ScmAccessException
     */
    private ScmRevisionIterator getLogData(final String[] command)
        throws IOException, InterruptedException, ScmAccessException {

        SimpleProcess proc = new SimpleProcess(command);

        // Gobble up stderr to prevent deadlock on Win32
        new StreamGobbler(proc.stderr()).start();

        // Loop through relevant revisions
        ScmRevisionIterator revisionIter = new RevisionIterator(proc.stdout());

        // Check for appropriate exit code and throw exception if necessary
        if (proc.waitFor() != 0) {
            throw new ScmAccessException("Could not access SCM system.");
        }

        return revisionIter;
    }
}
