/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.cvsPlugin;

import java.sql.Timestamp;

/** <p>History is a container class for a single CVS history record.</p>
 *
 * <p>A CVS history record follows the following general format:</p>
 * <code>CODE DATE USER [REVISION] [FILE] PATH == LOCAL_PATH</code>
 * @author Polina Gohshtein
 */
public class History {

    private String mOperation = null; // CVS operation
    private Timestamp mTimestamp = null; // Time of the operation
    private String mUser = null; // User that made the operation
    private String mRevision = null;
    private String mFile = null;
    private String mPath = null;

    /** A static method that checks whether the given operation
     * is a removal operation.
     * @param operation The operation
     * @return true if it is a removal operation
     */
    public static boolean isRemoveOperation(String operation) {
        if ("R".equals(operation)) {
            return true;
        }
        return false;
    }

    /** Default constructor **/
    public History() { }

    /** Creates a history record with the given operation,
     * timestamp, user, version, file and path.
     * @param op Operation
     * @param ts Timestamp
     * @param user User
     * @param version File version
     * @param file File name
     * @param path File path
     */
    public History(String op, Timestamp ts, String user,
            String version, String file, String path) {
        mOperation = op;
        mTimestamp = ts;
        mUser = user;
        mRevision = version;
        mFile = file;
        mPath = path;
    }

    /**
     * @return Returns the operation.
     */
    public String getOperation() {
        return mOperation;
    }
    /**
     * @param operation The operation to set.
     */
    public void setOperation(final String operation) {
        mOperation = operation;
    }
    /**
     * @return Returns the file.
     */
    public String getFile() {
        return mFile;
    }
    /**
     * @param file The file to set.
     */
    public void setFile(final String file) {
        mFile = file;
    }
    /**
     * @return Returns the path.
     */
    public String getPath() {
        return mPath;
    }
    /**
     * @param path The path to set.
     */
    public void setPath(final String path) {
        mPath = path;
    }
    /**
     * @return Returns the revision.
     */
    public String getRevision() {
        return mRevision;
    }
    /**
     * @param revision The revision to set.
     */
    public void setRevision(final String revision) {
        mRevision = revision;
    }
    /**
     * @return Returns the timestamp.
     */
    public Timestamp getTimestamp() {
        return mTimestamp;
    }
    /**
     * @param timestamp The timestamp to set.
     */
    public void setTimestamp(final Timestamp timestamp) {
        mTimestamp = timestamp;
    }

    /**
     * @return Returns the user.
     */
    public String getUser() {
        return mUser;
    }
    /**
     * @param user The user to set.
     */
    public void setUser(final String user) {
        mUser = user;
    }

    /**
     * @return qualified filename (path+name)
     */
    public String getQualifiedFilename() {
        if (mPath == null) {
            return null;
        }
        return mPath + "/" + mFile;
    }
    /**
     * @return Returns a string representation of the history record.
     */
    public String toString() {
        String strRepresentation = "Operation: " + mOperation
            + "; time: " + mTimestamp
            + "; user: " + mUser
            + "; path: " + getQualifiedFilename();
        if (mRevision != null) {
            strRepresentation += "; revision: " + mRevision;
        }
        return strRepresentation;
    }
    /** Compare this History object to the other History object.
     * @param other History object to compare
     * @return true if this object is equal to the other object
     */
    public boolean equals(final History other) {
        if (!mOperation.equals(other.getOperation())) {
            return false;
        }
        if (!mTimestamp.equals(other.getTimestamp())) {
            return false;
        }
        if (!mUser.equals(other.getUser())) {
            return false;
        }
        if (mRevision != null) {
            if (!mRevision.equals(other.getRevision())) {
                return false;
            }
        } else {
            if (other.getRevision() != null) {
                return false;
            }
        }
        // Check if the qualified names of the two history records
        // are the same
        String qname = null;
        if ((qname = getQualifiedFilename()) != null) {
            if (!qname.equals(other.getQualifiedFilename())) {
                return false;
            }
        } else {
            if (other.getQualifiedFilename() != null) {
                return false;
            }
        }
        return true;
    }
    /** hashCode.
     */
    public int hashCode () {
        return toString().hashCode();
    }
}
