/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.cvsPlugin;

import java.io.BufferedReader;
import java.io.Reader;
import java.sql.Timestamp;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Parser for the CVS history.
 * @author Polina Gohshtein
 */
public class HistoryParser implements Runnable {

    // A vector to hold the history records
    private Vector mHistoryRecords;
    private BufferedReader mReader;
    private String mCvsRoot;

    /** Regular expressions used for matching cvs history */
    static final String RE_CODE = "\\w";
    static final String RE_DATE = "[\\d-/\\.]+";
    static final String RE_TIME = "\\d+:\\d+";
    static final String RE_REVISION = "[\\d\\.]+";
    static final String RE_FILEOP = "[CGUPAMRW]";
    static final String RE_REVISIONLESS = "W";
    static final String TIME_SUFFIX = ":00.0";
    static final String NORECORDS = "No records selected.";

    /** Creates a history parser with a given reader.
     * @param aReader A buffered reader with the output of 'CVS history'.
     * @param aCvsRoot The cvs root from which the history was retrieved.
     **/
    public HistoryParser(final Reader aReader, final String aCvsRoot) {
        mReader = new BufferedReader(aReader);
        mHistoryRecords = new Vector();
        int beginIndex = aCvsRoot.lastIndexOf(":") + 1;
        mCvsRoot = aCvsRoot.substring(beginIndex);
    }

    /**
     * @see java.lang.Runnable#run()
     */
    public void run() {
        String line = null;
        try {
            while ((line = mReader.readLine()) != null) {
                if (!NORECORDS.equals(line)) {
                    parseNextHistoryLine(line);
                }
            }
        } catch (Exception e) {
            System.out.flush();
            e.printStackTrace();
            System.err.flush();
            System.exit(1);
        }
    }

    /** Parses a single line from CVS history.
     * @param aLine A line with a single CVS history record.
     * @throws Exception
     */
    private void parseNextHistoryLine(final String aLine) throws Exception {
        // Create a string tokenizer and break the line
        // down into tokens. Use space as a delimiter.
        StringTokenizer st = new StringTokenizer(aLine);
        History record = new History();
        // Parse and set the operation
        record.setOperation(parseOperation(st));
        // Parse and set the timestamp
        record.setTimestamp(parseTimestamp(st));
        // Parse and set the user
        record.setUser(parseString(st));
        // If the operation is associated with a specific file,
        // then parse the file name, revision and path.
        // Note: for now we do not want to store the operations that are
        // not associated with files because they add no value to our metrics.
        if (isOperationFileSpecific(record.getOperation())) {
            if (!isRevisionless(record.getOperation())) {
                record.setRevision(parseRevision(st));
            }
            record.setFile(parseString(st));
            record.setPath(mCvsRoot + "/" + parseString(st));

            // Add the history record to the set history records
            mHistoryRecords.add(record);
        }
    }

    /** Returns the cvs operation.
     * @param st The StringTokenizer for a single line of history
     * @return the operation
     * @throws Exception
     */
    private String parseOperation(final StringTokenizer st) throws Exception {
        if (st.hasMoreTokens()) {
            String code = st.nextToken();
            if (code.matches(RE_CODE)) {
                  return code;
            } else {
                // Throw exception
                throw new Exception("Wrong format of cvs history data.");
            }
        } else {
            // Throw exception
            throw new Exception("Wrong format of cvs history data.");
        }
    }

    /** Returns the timestamp of the cvs operation.
     * @param st The StringTokenizer for a single line of history
     * @return the timestamp
     * @throws Exception
     */
    private Timestamp parseTimestamp(final StringTokenizer st)
        throws Exception {

        String date = null;
        String time = null;
        // The first portion of the timestamp is the date.
        if (st.hasMoreTokens()) {
            date = st.nextToken();
            if (!date.matches(RE_DATE)) {
                // Throw exception
                throw new Exception("Wrong format of cvs history data.");
            }
        } else {
            // Throw exception
            throw new Exception("Wrong format of cvs history data.");
        }
        // The second portion of the timestamp is the time.
        if (st.hasMoreTokens()) {
            time = st.nextToken();
            if (!time.matches(RE_TIME)) {
                // Throw exception
                throw new Exception("Wrong format of cvs history data.");
            }
        } else {
            // Throw exception
            throw new Exception("Wrong format of cvs history data.");
        }
        // Skip the third portion. It is of the form +0000
        if (st.hasMoreTokens()) {
            st.nextToken();
        } else {
            // Throw exception
            throw new Exception("Wrong format of cvs history data.");
        }
        return Timestamp.valueOf(date + " " + time + TIME_SUFFIX);
    }

    /** Returns the next string in the StringTokenizer.
     * @param st The StringTokenizer for a single line of history
     * @return a string
     * @throws Exception
     */
    private String parseString(final StringTokenizer st) throws Exception {
        if (st.hasMoreTokens()) {
            return st.nextToken();
        } else {
            // Throw exception
            throw new Exception("Wrong format of cvs history data.");
        }
    }

    /** Returns the revision of the cvs file affected by the cvs operation.
     * @param st The StringTokenizer for a single line of history
     * @return the revision
     * @throws Exception
     */
    private String parseRevision(final StringTokenizer st) throws Exception {
        if (st.hasMoreTokens()) {
            String revision = st.nextToken();
            if (revision.matches(RE_REVISION)) {
                  return revision;
            } else {
                // Throw exception
                throw new Exception("Wrong format of cvs history data.");
            }
        } else {
            // Throw exception
            throw new Exception("Wrong format of cvs history data.");
        }
    }

    /** Checks if the given cvs operation is associated with a
     * specific file or not.
     * @param op The cvs operation
     * @return True if operation is associated with a specific file.
     * False otherwise.
     */
    private boolean isOperationFileSpecific(final String op) {
        if (op.matches(RE_FILEOP)) {
            return true;
        }
        return false;
    }

    /** Check if the given cvs operation is associated with a file revision.
     * @param op The cvs operation.
     * @return True if the given cvs operation is associated with
     * a specific revision of a file.
     */
    private boolean isRevisionless(final String op) {
        if (op.matches(RE_REVISIONLESS)) {
            return true;
        }
        return false;
    }

    /**
     * @return Returns a vector of CVS history records.
     */
    public Vector getHistory() {
        return mHistoryRecords;
    }
}
