/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.report;

import java.io.*;
import java.sql.*;
import java.util.*;

import org.jfree.data.general.*;
import org.jfree.chart.*;

import ca.utoronto.JCVSReport.*;
import ca.utoronto.JCVSReport.metric.*;

/**
 * <p>An abstract base class for representing a single graph.</p>
 *
 * <p>This class implements:</p>
 * <ul>
 * <li> loading properties of a graph from a resourcebundle
 * <li> accessor methods for common graph properties
 * <li> writing graphs to HTML files
 * <li> executing an SQL query for gathering the graph information
 * <li> store and retreive metric values
 * </ul>
 *
 * @author David James
 */
public abstract class BasicGraph implements Graph {
    protected ResourceBundle props;
    protected String id;
    protected MetricDatabase database;
    protected boolean perDeveloper = false;
    protected boolean incremental = false;

    /** Initialize Graph from properties file
     * @param database Where to get our data
     * @param id A unique ID for this graph
     * @param props Properties of this graph
     *  */
    public void init(final MetricDatabase database, final String id,
            final ResourceBundle props) throws Exception {
        this.props = props;
        this.id = id;
        this.database = database;
        this.perDeveloper = !(getProperty("perDeveloper", "no").equals("no"));
        this.incremental = !(getProperty("incremental", "no").equals("no"));
    }

    /** Write a chart to disk and link to it in an HTML file
     * @param output Where to write the HTML report
     * @param directory Where to store images
     * */
    public void write(final PrintWriter output, final File directory)
            throws Exception {

        // Save chart as a JPEG
        String filename = directory.getAbsolutePath() + "/" + id + ".jpg";
        int height = getHeight(), width = getWidth();
        ChartRenderingInfo info = new ChartRenderingInfo();
        ChartUtilities.saveChartAsJPEG(new File(filename), getChart(),
                width, height, info);

        // Write an image map in the HTML file
        ChartUtilities.writeImageMap(output, id, info);

        // Output HTML tag for chart
        output.println("<img border=\"0\" src=\"" + id + ".jpg\" width=\""
                + width + "\" height=\"" + height + "\" usemap=\"#"
                + id + "\">");

    }

    /** Build a dataset object from an SQL query */
    protected abstract Dataset getDataset() throws Exception;

    /** Build a chart from a dataset */
    protected abstract JFreeChart getChart() throws Exception;

    /** Get a property from the props resource bundle
     * @param name The name of the property */
    protected String getProperty(final String name) {
        return props.getString(name);
    }

    /** Get a property from the props resource bundle
     * (return defaultValue if the property does not exist)
     * @param name The name of the property
     * @param defaultValue The default value for the property (we return this
     * if the property does not exist) */
    protected String getProperty(final String name,
            final String defaultValue) {
        try {
            return props.getString(name);
        } catch (MissingResourceException ex) {
            return defaultValue;
        }
    }

    /** Get the title of the object */
    public String getTitle() {
        String title = getProperty("title");
        if (perDeveloper) {
            title += " per Developer";
        }
        return title;
    }

    /** Get the desired height of the chart */
    protected int getHeight() {
        return new Integer(getProperty("height", "300")).intValue();
    }

    /** Get the desired width of the chart */
    protected int getWidth() {
        return new Integer(getProperty("width", "500")).intValue();
    }

    /** Get the label for the x-axis */
    protected String getXLabel() {
        return getProperty("xlabel", "Date");
    }

    /** Get the label for the y-axis */
    protected String getYLabel() {
        return getProperty("ylabel", getProperty("metrics"));
    }

    /** Get the key attribute of a MetricRevision object.
     * In this case, our key attribute is 'MetricName'
     * @param revision The MetricRevision object
     * @return the key attribute
     */
    protected String getKey(MetricRevision revision) {
        return revision.getMetricName();
    }


    /** Set whether we should report this metric per developer
     * @param value true or false */
    public void setPerDeveloper(boolean value) {
        perDeveloper = value;
    }

    /** Get the label of a series on a graph from a MetricRevision object.
     * By default, we use the title of the metric as the label
     * for the series.
     *
     * @param revision the MetricRevision object
     * @return the series label
     */
    protected String getSeriesLabel(MetricRevision revision) {
        try {
            return MetricSet.metricTitle(revision.getMetricName());
        } catch (MissingResourceException ex) {
            return revision.getMetricName();
        }

    }

    /** Join a list of strings together delimited by the specified token.
     * This is a helpful utility method used by executeQuery to join
     * the list of metrics together.
     *
     * @param token The delimiter
     * @param strings The list of strings
     * @return
     */
    protected String join(String token, String[] strings) {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < strings.length - 1; i++) {
            buf.append(strings[i]);
            buf.append(token);
        }

        buf.append(strings[strings.length - 1]);

        return buf.toString();
    }

    /** Execute our data gathering query
     * If the user specifies an SQL query in the properties file
     * for this graph, then we will use it. Otherwise, we will
     * generate a query using the provided list of metrics from the
     * properties file.
     */
    protected ResultSet executeQuery() throws SQLException {
        try {
            return database.executeQuery(getProperty("sql"));
        } catch (MissingResourceException ex) {
            String metrics = join("','", getMetricNames());
            return database.executeQuery(
                "SELECT metric_name, file, tstamp, metric_value, developer"
                + " FROM MetricRevision"
                + " WHERE metric_name IN ('" + metrics + "')"
                + " ORDER BY tstamp");
        }
    }

    // Maps for storing the metric values associated with each
    // file and developer
    protected MetricRevisionMap fileMap =
        new MetricRevisionMap(MetricRevision.byFile());
    protected MetricRevisionMap devMap =
        new MetricRevisionMap(MetricRevision.byDeveloper());

    /** Store the value associated with the current metric result
     * so that it can be retreived later
     * @param revision The current metric result */
    protected void storeMetricValue(MetricRevision revision) {
        double diff = revision.getMetricValue();
        if (!incremental) {
            diff -= fileMap.getDouble(revision);
        }
        devMap.putDouble(revision, devMap.getDouble(revision) + diff);
        fileMap.putDouble(revision, revision.getMetricValue());
    }

    /** Get the stored metric value associated with the specified MetricRevision
     * @param revision The specified MetricRevision */
    protected double getStoredMetricValue(MetricRevision revision) {
        if (perDeveloper) {
            return devMap.getDouble(revision);
        } else {
            return devMap.sum(revision.getMetricName());
        }
    }

    /** Get a list of the metrics supported by this graph */
    public String[] getMetricNames() {
        return getProperty("metrics").split("\\W");
    }


 }
