/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.report;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import ca.utoronto.JCVSReport.metric.*;
import ca.utoronto.JCVSReport.*;

/**
 * Create new graph objects based on a descriptive string
 * @author David James
 */
public class GraphBuilder {

    String graphString;
    String graphName;
    String graphClass = "TimeSeriesGraph";
    ResourceBundle props;
    boolean perDeveloper;

    /** Create a new GraphLoader object for creating
     * graphs described by the specified graphString
     * @param graphString The graph name specified in the properties file
     */
    public GraphBuilder(String graphString) {
        this.graphString = graphString;

        String[] graphDesc = graphString.split("-");

        // Calculate Graph Name
        graphName = graphDesc[0];

        // Get the properties associated with this graph
        props = getGraphProperties(graphName);

        // Calculate Graph Class
        if (graphDesc.length > 1) {
            graphClass = graphDesc[1];

            // Should the graph display per developer?
            perDeveloper = graphClass.startsWith("PerDeveloper");

            // If so, then we'll need to remove the PerDeveloper prefix
            if (perDeveloper) {
                graphClass = graphClass.replaceFirst("PerDeveloper", "");
                if (graphClass.equals("")) {
                    graphClass = "PieGraph";
                }
            }
        } else {
            try {
                graphClass = props.getString("class");
            } catch (MissingResourceException ex) {
                graphClass = "TimeSeriesGraph";
            }
        }
    }

    /** Return the graph properties file for graphName
     * @param graphName the name of the graph */
    ResourceBundle getGraphProperties(final String graphName) {
        final String pkg = getClass().getPackage().getName();
        try {
            return ResourceBundle.getBundle(pkg + ".properties." + graphName);
        } catch (MissingResourceException ex) {
            return new MetricGraphProperties(graphName);
        }
    }

    /** Build and initialize a new graph object
     * @param database The database the graph should use
     */
    public Graph build(MetricDatabase database) throws Exception {

        Graph graph;

        // Load in the .properties file for this report
        try {
            graph = (Graph) Class.forName(graphClass).newInstance();
        } catch (ClassNotFoundException ex) {
            final String pkg = getClass().getPackage().getName() + ".";
            graph = (Graph) Class.forName(pkg + graphClass).newInstance();
        }

        // Initialize our report with the specified properties
        graph.init(database, graphString, props);

        // If the user requested a per Developer graph, make it so
        if (perDeveloper) {
            graph.setPerDeveloper(true);
        }

        return graph;
    }


}
