/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.report;

import java.util.*;
import java.io.*;
import ca.utoronto.JCVSReport.metric.*;
import ca.utoronto.JCVSReport.*;


/**
 * Write a set of graphs out to an HTML file
 *
 * @author David James
 */
public class Report {

    Vector graphs = new Vector();
    MetricSet metrics;
    MetricDatabase database;

    /** Initialize a new Report object
     * @param database Where to get our statistics */
    public Report(final MetricDatabase database) {
        this.database = database;
        metrics = new MetricSet(database);
    }


    /** Add a graph to this report
     * @param graphName The name of the graph we want to add */
    public void addGraph(final String graphName) throws Exception {

        // Instantiate graph class
        GraphBuilder graphBuilder = new GraphBuilder(graphName);
        Graph graph = graphBuilder.build(database);

        String[] metricNames = graph.getMetricNames();
        // Add necessary metrics to our set of metrics
        for (int i = 0; i < metricNames.length; i++) {
            metrics.addMetric(metricNames[i]);
        }

        // Add the specified report to our list of reports
        graphs.add(graph);
    }



    /** Add in a set of graphs
     * @param graphNames The names of the graphs we want to add */
    public void addGraphs(final String[] graphNames) throws Exception {
        for (int i = 0; i < graphNames.length; i++) {
            addGraph(graphNames[i]);
        }
    }

    /** Write reports to an output directory
     * @param output Where to write the HTML report
     * @param directory Where to store images
     * */
    public void write(final String output, final String directory)
            throws Exception {
        FileOutputStream reportFile = new FileOutputStream(output);
        write(new PrintWriter(reportFile), new File(directory));
    }


    /** Write reports to an output directory
     * @param output Where to write the HTML report
     * @param directory Where to store images
     * */
    public void write(final PrintWriter output, final File directory)
            throws Exception {

        // Output HTML header
        output.println("<html><head><title>JCVSReport</title></head><body>");

        // Output title
        output.println("<h1><a name='top'>Reports</a></h1>");

        // Output table of contents
        int id = 1;
        output.println("<ul>");
        for (Iterator it = graphs.iterator(); it.hasNext();) {
            Graph report = (Graph) it.next();
            output.println("<li><a href='#graph" + id + "'>"
                    + report.getTitle() + "</a></li>");
            id++;
        }
        output.println("</ul>");

        // Output each report
        id = 1;
        for (Iterator it = graphs.iterator(); it.hasNext();) {
            Graph report = (Graph) it.next();

            // Output title and header for this report
            output.println("<hr><h1><a name='graph" + id + "'>"
                    + report.getTitle() + "</a></h1>");

            // Output report
            report.write(output, directory);

            output.println("<br><a href='#top'>Back to top</a>");

            id++;
        }
        output.println("</body></html>");

        // Write the HTML file out right away
        output.flush();
    }


    /** Process the contents of a revision of a file with each metric
     * @param revisionId A unique ID for this revision of this file
     * @param contents The contents of the file as a string
     * @throws Exception if this revision cannot be processed
     * */
    public void processRevision(final String revisionId,
            final String contents) throws Exception {
        metrics.processRevision(revisionId, contents);
    }

    /**
     * Get a list of the graphs
     */
    public Graph[] getGraphs() {
        return (Graph[]) graphs.toArray();
    }

}
