/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.report;

import ca.utoronto.JCVSReport.metric.*;

import org.jfree.chart.*;
import org.jfree.data.xy.*;
import org.jfree.data.time.*;
import org.jfree.data.general.*;
import java.sql.*;
import java.util.*;

/**
 * A graph that displays the values of one or more metrics over time.
 *
 *
 * @author David James
 */
 public class TimeSeriesGraph extends BasicGraph {

     // The list of series
     Map seriesMap = new HashMap();

     // Collect all of our series in one seriesCollection
     TimeSeriesCollection seriesCollection = new TimeSeriesCollection();

     HashMap lastResultMap = new HashMap();

     /** Add the current result to the current series
      * @param series The current series
      * @param revision The current result
      */
     void addRevisionToSeries(TimeSeries series, MetricRevision revision) {

         // Store the current metricValue in our hash
         storeMetricValue(revision);

         /* Add the datapoint to the graph */
         series.addOrUpdate(revision.getDay(), getStoredMetricValue(revision));
     }


     /** Fill the interim days in the series according to the MetricRevision
      * @param series The series to fill
      * @param revision The latest revision
      */
     void fillInterimDays(TimeSeries series, MetricRevision revision)
             throws SQLException {

         // Get the old value for this series
         double value = getStoredMetricValue(revision);

         String key = getKey(revision);
    
         if (lastResultMap.containsKey(key)) {
             // Get the last day
             RegularTimePeriod lastDay = getLastRevision(key).getDay();

             // Fill interim days from lastDay to currentDay with old value
             fillInterimDays(series, lastDay, revision.getDay(), value);
         } else {
             // Fill previous day with zero
             fillInterimDays(series, revision.getDay().previous().previous(), 
                     revision.getDay(), 0);        
         }
     }

     /** Fill from fromDate to toDate (exclusive) with the specified value
      * @param series The series to fill
      * @param fromDate The day before the first day you want to fill
      * @param toDate The day after the last day you want to fill
      * @param value The value you want to fill it with
      **/
     void fillInterimDays(TimeSeries series, RegularTimePeriod fromDate,
             RegularTimePeriod toDate, double value) throws SQLException {

         /* Ensure that the lastDay isn't after the current day */
         if (fromDate.compareTo(toDate) > 0) {
             throw new SQLException(
                 "Query must be sorted by tstamp field"
             );
         }

         // Fill in the interim days with data
         fromDate = fromDate.next();
         while (fromDate.compareTo(toDate) < 0) {
             series.addOrUpdate(fromDate, value);
             fromDate = fromDate.next();
         }
     }

     /** Get the current series for a given metric result
      * @param revision The given metric result */
     TimeSeries getSeries(MetricRevision revision) throws SQLException {

         // Get a unique identifier for this series
         String key = getKey(revision);

         TimeSeries series;
         // If we've seen this series before...
         if (seriesMap.containsKey(key)) {

             // Get the current TimeSeries
             series = (TimeSeries) seriesMap.get(key);

         } else {

             // Get title for this metric
             String seriesLabel = getSeriesLabel(revision);

             // Create new series
             series = new TimeSeries(seriesLabel);
             seriesMap.put(key, series);

             // Add this series to the overall seriesCollection
             seriesCollection.addSeries(series);
         }

         return series;
     }

     /** Fill the end of the chart with values for all of the series up
      * to toDate
      * @param toDate the date to which you want to fill the chart
      */
     void fillRemainingValuesForAllSeries(RegularTimePeriod toDate)
             throws SQLException {

         Iterator it = lastResultMap.keySet().iterator();

         /* Fill in remaining values in chart */
         while (it.hasNext()) {

             // Get the key and the series associated with it
             String key = (String) it.next();
             TimeSeries series = (TimeSeries) seriesMap.get(key);

             // Get the last result from this series
             MetricRevision lastRevision = getLastRevision(key);

             // Get the value of the last result and the last day
             double value = getStoredMetricValue(lastRevision);
             RegularTimePeriod lastDay = lastRevision.getDay();

             // Fill days from the last
             fillInterimDays(series, lastDay, toDate.next(), value);
         }
     }


     /** Get the last result for the specified series
     * @param key a unique ID for the series
     * @return
     */
    private MetricRevision getLastRevision(String key) {
        return (MetricRevision) lastResultMap.get(key);
    }


    /** Build a seriesCollection object from an SQL query
      * NOTE: We assume that the SQL query returns values sorted by date */
     protected Dataset getDataset() throws Exception {

         // The date associated with the current row
         RegularTimePeriod currentDay = new Day();

         // Execute the query and loop through the keys
         ResultSet metricRevisions = executeQuery();

         // Initialize MetricRevision
         MetricRevision revision = null;

         // Loop through all the metric revisions that match our query
         while (metricRevisions.next()) {

             // Create object which describes this MetricRevision
             revision = new MetricRevision(metricRevisions);

             // Get the current series
             TimeSeries series = getSeries(revision);

             // Fill interim days for this series with datavalues
             fillInterimDays(series, revision);

             // Add the datapoint to the series
             addRevisionToSeries(series, revision);

             // Save result in maps for later retrieval
             lastResultMap.put(getKey(revision), revision);
         }

         /** If we have results, then fill the remaining values in the chart */
         if (revision != null) {
             fillRemainingValuesForAllSeries(revision.getDay());
         }

         return seriesCollection;
     }


     /** Build a chart from a seriesCollection */
     protected JFreeChart getChart() throws Exception {
         return ChartFactory.createTimeSeriesChart
         (getTitle(),                  // Title
          getXLabel(),                 // X-Axis label
          getYLabel(),                 // Y-Axis label
          (XYDataset) getDataset(),    // Dataset
          true,                        // Show legend
          true,                        // Use tooltips
          false                        // Configure chart to generate URLs?
         );
     }

     /** Get the title of the graph object */
     public String getTitle() {
         return super.getTitle() + " over time";
     }

    /** Get the key attribute of a MetricRevision object.
     * If our metric is PerDeveloper, we use the developer
     * as the key. Otherwise, we use the metric name.
     * @param revision The MetricRevision object
     * @return the key attribute
     */
     protected String getKey(MetricRevision revision) {
        if (perDeveloper) {
            return revision.getDeveloper();
        } else {
            return revision.getMetricName();
        }
     }
         
    /** Get the label of a series on a graph from a MetricRevision object.
     * By default, we use the same series label as the BasicGraph.
     * If the metric is PerDeveloper, then we use the developer name
     * as the series label.
     *
     * @param revision the MetricRevision object
     * @return the series label
     */
    protected String getSeriesLabel(MetricRevision revision) {
        if (perDeveloper) {
            return revision.getDeveloper();
        } else {
            return super.getSeriesLabel(revision);
        }
    }

     
     
}
