/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.SimpleProcess;

import java.io.*;



/**
 * SimpleProcess: A simpler interface for running external processes
 * @author David James
 */
public class SimpleProcess {
    Process proc;
    BufferedReader stdout;
    BufferedReader stderr;

    /** Run a process and return its result as a string
     * @param command The command to run */
    public static String run(final String[] command) throws IOException,
            InterruptedException {
        SimpleProcess proc = new SimpleProcess(command);

        // Gobble up stderr to prevent deadlock
        new StreamGobbler(proc.stderr()).start();

        // Read input lines one at a time from subprocess
        StringWriter writer = new StringWriter();
        BufferedReader reader = proc.stdout();
        String line;
        while ((line = reader.readLine()) != null) {
            writer.write(line);
            writer.write("\n");
        }

        // Check for appropriate exit code and throw exception if necessary
        if (proc.waitFor() != 0) {
            throw new InterruptedException("Non-zero exit code");
        }

        // Return as string
        return writer.toString();
    }


    /** Create a new SimpleProcess object
     * @param command The command to run */
    public SimpleProcess(final String[] command) throws IOException {
        proc = Runtime.getRuntime().exec(command);
    }

    /** Access the stderr stream for the process */
    public BufferedReader stderr() {
        if (stderr == null) {
            stderr = new BufferedReader(
                    new InputStreamReader(proc.getErrorStream()));
        }
        return stderr;
    }

    /** Access the stdout stream for the process */
    public BufferedReader stdout() {
        if (stdout == null) {
            stdout = new BufferedReader(
                    new InputStreamReader(proc.getInputStream()));
        }
        return stdout;
    }

    /** Wait for the process to exit */
    public int waitFor() throws InterruptedException {
        return proc.waitFor();
    }

}
