/*
    Bloof - visualize the evolution of your software project
    Copyright ( C ) 2003  Lukasz Pekacki <lukasz@pekacki.de>
    http://bloof.sf.net/

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    $RCSfile: Database.java,v $
    Created on $Date: 2005/03/14 10:14:03 $
*/
package net.sf.bloof.db;

import net.sf.bloof.metrics.TimeInterval;
import net.sf.bloof.scm.FileState;
import net.sf.bloof.scm.ScmAccess;
import net.sf.bloof.scm.ScmRevision;
import net.sf.bloof.scm.ScmRevisionIterator;
import net.sf.bloof.scm.cvsplugin.CvsAccess;
import net.sf.bloof.scm.cvsplugin.InvalidRepositoryLocationException;
import net.sf.bloof.scm.cvsplugin.LoginDetails;
import net.sf.bloof.scm.cvsplugin.RepositoryLocation;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

import java.util.Date;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;


/**
 * This class manages the interaction with the database via JDBC Connection
 * @author Lukasz Pekacki <pekacki@users.sourceforge.net>
 * @version $Id: Database.java,v 1.3 2005/03/14 10:14:03 g2polgoh Exp $
 * @todo Externalize SQL statements
 */
public class Database {
    /* DDL script files */
    protected static final String CREATE_TABLE_FILE = "database.ddl";

    /* DDL script files */
    protected static final String CREATE_INDEX_FILE = "indices.ddl";

    /* DDL script files */
    protected static final String DROP_TABLE_FILE = "droptables.ddl";
    protected static Logger sLogger = Logger.getLogger(Database.class.getName());
    protected static final String SQL_ADD_DEVELOPER = "INSERT INTO Developer ( login ) VALUES ( ? )";
    protected static final String SQL_ADD_FILE = "INSERT INTO File ( pathname,name ) VALUES ( ?,? )";
    protected static final String SQL_ADD_REVISION =
        "INSERT INTO Revision ( file,tstamp,developer,version,lines_add,lines_del, description ) " +
        "VALUES ( ?,?,?,?,?,?,? )";
    protected static final String SQL_ADD_FILE_DELETION = "INSERT INTO DeletedFile (file,tstamp) VALUES (?,?)";
    protected static final String SQL_GET_DEVELOPER = "SELECT login FROM Developer ORDER BY login";
    protected static final String SQL_GET_FILES = "SELECT pathname FROM File ORDER BY pathname";
    protected static final String SQL_GET_PROJECTDATA = "SELECT * FROM Proejct";
    protected static final String SQL_GET_TIME_BOUNDARY = "SELECT MIN( tstamp ), MAX( tstamp ) FROM Revision";

    /* internal attributes */
    protected Connection mConnection;
    protected Hashtable mDeveloperNames;
    protected Hashtable mFileNameMaxRevisionDate;
    protected PreparedStatement mPrepAddFileDeletionStatement;
    protected PreparedStatement mPreparedAddDeveloperStatement;
    protected PreparedStatement mPrepFileAddStatement;
    protected PreparedStatement mPrepRevisionAddStatement;
    protected Statement mStatement;

    /* Database population attributes */
    private int mSumRevisions = 0;
    private TimeInterval mWholeTimespan;

    /**
    * Sets the connection and fills the hashtables with the already
    * stored keys in the database
    * @param aConnection The connection to set
    * @throws SQLException on connection error
    */
    public Database(Connection aConnection) throws SQLException {
        mConnection = aConnection;
        mStatement = mConnection.createStatement();
        mPrepFileAddStatement = mConnection.prepareStatement(SQL_ADD_FILE);
        mPrepRevisionAddStatement = mConnection.prepareStatement(SQL_ADD_REVISION);
        mPreparedAddDeveloperStatement = mConnection.prepareStatement(SQL_ADD_DEVELOPER);
        mPrepAddFileDeletionStatement = mConnection.prepareStatement(SQL_ADD_FILE_DELETION);
        fillDeveloperHash();
        fillFilesHash();
    }

    /**
     * Fetches the next query from the input.  Returns null if no more queries
     * available.
     */
    private static String nextQuery(Reader aInputReader)
        throws IOException {
        StringBuffer query = new StringBuffer();
        int c = aInputReader.read();
        boolean command = false;

        while (c != -1) {
            if ((command == true) ||
                    ((c != ' ') && (c != '\n') && (c != '\r') && (c != '\t'))) {
                command = true;
            }

            query.append((char) c);

            if (c == ';') {
                return new String(query);
            }

            c = aInputReader.read();
        }

        if (command == false) {
            return null;
        }

        return new String(query);
    }

    /**
     * Evaluates the input script and outputs the result to the output stream.
     * Returns the list of Connections established.
     * @param aScriptFileName name of the Script file
     * @param aDbConnection connectin to the database
     * @throws IOException on error
     */
    protected static void proccessDBScript(String aScriptFileName,
        Connection aDbConnection) throws IOException {
        // Read in the query.  The query is everything up to the ';' character
        // which denotes the end of an SQL command.
        // ISSUE: It currently does not recognise ';' if used inside a string...
        Reader inputReader = new BufferedReader(new InputStreamReader(
                    Database.class.getResourceAsStream(aScriptFileName)));

        Statement statement;

        try {
            statement = aDbConnection.createStatement();
        } catch (SQLException e) {
            sLogger.warning("SQL Error creating statement: " + e.getMessage());

            return;
        }

        String query = nextQuery(inputReader);

        while (query != null) {
            try {
                String command = query.substring(0, query.length() - 1).trim()
                                      .toLowerCase();

                if (command.startsWith("//")) {
                    sLogger.fine(command);
                } else {
                    sLogger.finest(query.trim());
                    statement.execute(query);
                }
            } catch (SQLException e) {
                sLogger.warning("SQL Error running query ( \n" + query +
                    "\n )\nError: " + e.getMessage());
            }

            // Fetch the next query
            query = nextQuery(inputReader);
        }

        inputReader.close();
    }

    protected void addDeletedFile(ScmRevision aRevision)
        throws SQLException {
        mPrepAddFileDeletionStatement.setString(1,
            aRevision.getQualifiedFilename());
        mPrepAddFileDeletionStatement.setTimestamp(2,
            new Timestamp(aRevision.getDate().getTime()));
        mPrepAddFileDeletionStatement.executeUpdate();
    }

    /**
     * Add a file into the database
     * @param aRevision associated Revision
     */
    protected void addFile(ScmRevision aRevision) throws SQLException {
        mPrepFileAddStatement.setString(1, aRevision.getQualifiedFilename());
        mPrepFileAddStatement.setString(2, aRevision.getFileName());
        mPrepFileAddStatement.executeUpdate();
    }

    /**
     * Add a developer to the database
     * @param aDeveloperName name of the developer
     */
    protected void addNewDeveloper(String aDeveloperName)
        throws SQLException {
        mPreparedAddDeveloperStatement.setString(1, aDeveloperName);
        mPreparedAddDeveloperStatement.executeUpdate();
    }

    /**
     * Adds this revision to the database
     * @param aRevision
     */
    protected void addRevision(ScmRevision aRevision) throws SQLException {
        mSumRevisions++;
        mPrepRevisionAddStatement.setString(1, aRevision.getQualifiedFilename());
        mPrepRevisionAddStatement.setTimestamp(2,
            new Timestamp(aRevision.getDate().getTime()));
        mPrepRevisionAddStatement.setString(3, aRevision.getAuthor());
        mPrepRevisionAddStatement.setString(4, aRevision.getRevisionName());
        mPrepRevisionAddStatement.setInt(5, aRevision.getAdded());
        mPrepRevisionAddStatement.setInt(6, aRevision.getRemoved());
        mPrepRevisionAddStatement.setString(7, aRevision.getComment());
        mPrepRevisionAddStatement.executeUpdate();

        if (aRevision.getFileState() == FileState.DEAD) {
            addDeletedFile(aRevision);
        }
    }

    /**
     * Closes the Database
     * @throws SQLException on error
     * @throws IOException on error
     */
    public void close() throws SQLException, IOException {
        McKoiControl.close();
        PostgresControl.close();
        mConnection.close();
    }

    /**
     * Returns the ResultSet for a SQL Query
     * @param aSqlQuery SQL statement to run on the database
     * @return ResultSet for the specified SQL query
     * @throws SQLException on sql error
     */
    public ResultSet executeQuery(String aSqlQuery) throws SQLException {
        sLogger.fine("Running query:" + aSqlQuery);

        return mStatement.executeQuery(aSqlQuery);
    }

    protected void fillDeveloperHash() throws SQLException {
        mDeveloperNames = new Hashtable();

        ResultSet developers = mStatement.executeQuery(
                "SELECT login from Developer");

        while (developers.next()) {
            String login = developers.getString("login");

            if (login == null) {
                return;
            }

            mDeveloperNames.put(login, login);
        }
    }

    protected void fillFilesHash() throws SQLException {
        mFileNameMaxRevisionDate = new Hashtable();

        ResultSet files = mStatement.executeQuery("SELECT pathname from File");
        HashSet filesSet = new HashSet();

        while (files.next()) {
            String qualiname = files.getString("pathname");
            filesSet.add(qualiname);
        }

        for (Iterator iter = filesSet.iterator(); iter.hasNext();) {
            String qualiname = (String) iter.next();
            String query =
                "SELECT max(tstamp) as maxdate from Revision where file = '" +
                qualiname + "' ";
            ResultSet maxDateRes = mStatement.executeQuery(query);

            maxDateRes.next();

            Timestamp t = maxDateRes.getTimestamp(1);

            if (t != null) {
                Date maxDate = new Date(maxDateRes.getTimestamp(1).getTime());
                mFileNameMaxRevisionDate.put(qualiname, maxDate);
            }
        }
    }

    /**
     * Formats the ResultSet as plain mono-spaced text and outputs the result as
     * String
     * @param aResultSet input set
     * @return String return
     * @throws SQLException on error
     */
    public String formatResultSet(ResultSet aResultSet)
        throws SQLException {
        StringBuffer sb = new StringBuffer();
        ResultSetMetaData metaData = aResultSet.getMetaData();

        // Maximum widths of each column.
        int[] maxWidths = new int[metaData.getColumnCount()];
        Vector[] data = new Vector[metaData.getColumnCount()];

        for (int i = 0; i < data.length; ++i) {
            data[i] = new Vector();
        }

        int rowCount = 0;

        for (int i = 0; i < data.length; ++i) {
            String str = metaData.getColumnLabel(i + 1);
            maxWidths[i] = Math.max(str.length(), maxWidths[i]);
        }

        // Read in the data for the result set,
        while (aResultSet.next()) {
            for (int i = 0; i < data.length; ++i) {
                Object ob = aResultSet.getObject(i + 1);
                String str = "NULL";

                if (ob != null) {
                    str = ob.toString();
                }

                data[i].addElement(str);
                maxWidths[i] = Math.max(str.length(), maxWidths[i]);
            }

            ++rowCount;
        }

        // Output the data we stored
        String[] line = new String[data.length];

        sb.append(formatResultWriteBreak(maxWidths));

        for (int n = 0; n < line.length; ++n) {
            line[n] = metaData.getColumnLabel(n + 1);
        }

        sb.append(formatResultSetRow(maxWidths, line));
        sb.append(formatResultWriteBreak(maxWidths));

        for (int i = 0; i < rowCount; ++i) {
            for (int n = 0; n < line.length; ++n) {
                line[n] = (String) data[n].elementAt(i);
            }

            sb.append(formatResultSetRow(maxWidths, line));
        }

        sb.append(formatResultWriteBreak(maxWidths));

        return sb.toString();
    }

    private String formatResultSetRow(int[] aWidths, String[] aCols) {
        StringBuffer sb = new StringBuffer();
        sb.append('|');

        for (int i = 0; i < aWidths.length; ++i) {
            String str = aCols[i];
            sb.append(' ');
            sb.append(str);

            // Write padding
            int wid = (aWidths[i] + 1) - str.length();

            for (int n = 0; n < wid; ++n) {
                sb.append(' ');
            }

            sb.append('|');
        }

        return sb.toString();
    }

    private String formatResultWriteBreak(int[] aWidths) {
        String s = new String();
        s = "+";

        for (int i = 0; i < aWidths.length; ++i) {
            int wid = aWidths[i] + 2;

            for (int n = 0; n < wid; ++n) {
                s = s + "+";
            }

            s = s + "+";
        }

        return s;
    }

    /**
     * Returns the StringIterator of all Developers
     * ordered by name
     * @return StringIterator of a Select on all Developers
     * @throws SQLException on sql error
     */
    public StringIterator getDevelopersOrderedByName()
        throws SQLException {
        Statement stmt = mConnection.createStatement();

        return new StringIterator(stmt.executeQuery(SQL_GET_DEVELOPER));
    }

    /**
     * Returns the StringIterator of all Files ( pathname )
     * ordered by pathname
     * @return StringIterator of a Select on all Files ( pathname )
     * ordered by pathname
     * @throws SQLException on sql error
     */
    public StringIterator getFilesOrderdByPath() throws SQLException {
        Statement stmt = mConnection.createStatement();

        return new StringIterator(stmt.executeQuery(SQL_GET_FILES));
    }

    /**
     * Returns a result set that point on the first row. This method is
     * a shortcut for fetching a SELECT from a database that has only
     * one row as result, e.g. "SELECHT COUNT( * ) FROM table".
     * If the request has no result rows, null is returned
     * @param aSqlQuery the query specified
     * @return ResultSet
     * @throws SQLException on sql error
     */
    public ResultSet getOnlyRowResultSet(String aSqlQuery)
        throws SQLException {
        Statement stmt = mConnection.createStatement();
        sLogger.fine("Running one row query:" + aSqlQuery);

        ResultSet rs = stmt.executeQuery(aSqlQuery);

        if (rs.next()) {
            return rs;
        } else {
            return null;
        }
    }

    /**
     * Returns the SCM Access to the project
     * @return SCM Access to the project
     * @throws SQLException on db error
     */
    public ScmAccess getProjectScmAccess() throws SQLException {
        ResultSet res = getOnlyRowResultSet(SQL_GET_PROJECTDATA);
        String pname = res.getString("name");
        String login = res.getString("login");
        String password = res.getString("passwd");
        String location = res.getString("location");
        String module = res.getString("module");
        RepositoryLocation relo = null;

        try {
            relo = new RepositoryLocation(location);
        } catch (InvalidRepositoryLocationException ire) {
            ire.printStackTrace();
        }

        LoginDetails logD = new LoginDetails(login, password);
        CvsAccess cvs = new CvsAccess(relo, logD, module, pname);

        return cvs;
    }

    /**
     * Returns the maximum TimeInterval on the whole project
     * @return TimeInterval of the whole project
     * @throws SQLException on sql error
     */
    public TimeInterval getTimespanBoundaries() throws SQLException {
        if (mWholeTimespan == null) {
            ResultSet result = getOnlyRowResultSet(SQL_GET_TIME_BOUNDARY);
            mWholeTimespan = new TimeInterval(result.getTimestamp(1),
                    result.getTimestamp(2));
        }

        return mWholeTimespan;
    }

    /**
     * Populates the database with the revisions of the specified
     * RevisionIterator
     * Already existing revisions will not be updated in the database
     * @param aRevisonIter Iterator for revisions
     * @throws SQLException if errors occur
     */
    public void populateDatabase(ScmRevisionIterator aRevisonIter)
        throws SQLException {
        mConnection.setAutoCommit(false);
        mSumRevisions = 0;

        while (aRevisonIter.hasNext()) {
            ScmRevision revision = aRevisonIter.getNext();

            if (!mDeveloperNames.containsKey(revision.getAuthor())) {
                addNewDeveloper(revision.getAuthor());
                mDeveloperNames.put(revision.getAuthor(), revision.getAuthor());
            }

            if (!mFileNameMaxRevisionDate.containsKey(
                        revision.getQualifiedFilename())) {
                addFile(revision);
                mFileNameMaxRevisionDate.put(revision.getQualifiedFilename(),
                    new Date(0));
            }

            Date fileMaxDate = (Date) mFileNameMaxRevisionDate.get(revision.getQualifiedFilename());

            if (revision.getDate().after(fileMaxDate)) {
                addRevision(revision);
            }
        }

        mConnection.setAutoCommit(true);
    }

    /**
     * Updates a project
     * @param aRevisonIter iterator on the new revisions
     * @throws SQLException on database error
     */
    public void updateDatabase(ScmRevisionIterator aRevisonIter)
        throws SQLException {
        mConnection.setAutoCommit(false);
        mSumRevisions = 0;

        while (aRevisonIter.hasNext()) {
            ScmRevision revision = aRevisonIter.getNext();

            if (!mDeveloperNames.containsKey(revision.getAuthor())) {
                addNewDeveloper(revision.getAuthor());
                mDeveloperNames.put(revision.getAuthor(), revision.getAuthor());
            }

            if (!mFileNameMaxRevisionDate.containsKey(
                        revision.getQualifiedFilename())) {
                addFile(revision);
                mFileNameMaxRevisionDate.put(revision.getQualifiedFilename(),
                    revision.getDate());
            }

            Object o = (Object) mFileNameMaxRevisionDate.get(revision.getQualifiedFilename());
            Date fileMaxDate = (Date) o;

            if (revision.getDate().after(fileMaxDate)) {
                addRevision(revision);
            }
        }

        mConnection.setAutoCommit(true);
        mWholeTimespan = null;
    }
}
