/*
    Bloof - visualize the evolution of your software project
    Copyright ( C ) 2003  Lukasz Pekacki <lukasz@pekacki.de>
    http://bloof.sf.net/

    This file was part of JavaCVS by Nicholas Allen ( nallen@freenet.co.uk ).

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    $RCSfile: RepositoryLocation.java,v $
    Created on $Date: 2005/03/14 02:19:06 $
*/

package net.sf.bloof.scm.cvsplugin;

/**
 * Defines the location of a repository.
 * @author Nicholas Allen
 * @author Lukasz Pekacki
 * */

public class RepositoryLocation {

    /**
     * Creates a CVSRepositoryLocation object using the specified parameters
     * @param aMethod method of connection
     * @param aUserName login of the user
     * @param aHostName name of cvs host
     * @param aRepositoryPath path to CVS repository
     * @throws InvalidRepositoryLocationException if the specified String does not match
     */
    public RepositoryLocation(
        CvsConnectionMethod aMethod,
        String aUserName,
        String aHostName,
        String aRepositoryPath)
        throws InvalidRepositoryLocationException {
        if (aMethod == null) {
            throw new InvalidRepositoryLocationException("Method not specified");
        }

        if (!aMethod.equals(CvsConnectionMethod.LOCAL)
            && (aHostName == null || aHostName.trim().length() == 0)) {
            throw new InvalidRepositoryLocationException("Host name not specified");
        }

        if (aRepositoryPath == null || aRepositoryPath.trim().length() == 0) {
            throw new InvalidRepositoryLocationException("Repository path not specified");
        }

        mConnectionMethod = aMethod;
        mUserName = aUserName;
        mHostName = aHostName;
        mRepositoryPath = aRepositoryPath;
    }

    /**
     * Creates a CVSRepositoryLocation object using sLocation as the location.
     * @param aLocation the location of the repository. This is of the form:<p>
     * <pre>:method:user@host:/path/to/repository</pre>
     * @throws InvalidRepositoryLocationException if the specified String does not match
     * */
    public RepositoryLocation(String aLocation) throws InvalidRepositoryLocationException {
        if (aLocation == null) {
            throw new InvalidRepositoryLocationException(
                "The repository location is null:" + aLocation);
        }

        int nEndOfConnectionMethod = -1;

        // Check if repository connection method is explicitly defined

        if (aLocation.charAt(0) == ':') {
            nEndOfConnectionMethod = aLocation.indexOf(':', 1);

            if (nEndOfConnectionMethod == -1) {
                throw new InvalidRepositoryLocationException(
                    "Connection method is not" + "contained between two ':' characters");
            }

            String sConnectionMethod = aLocation.substring(1, nEndOfConnectionMethod);

            if (sConnectionMethod.equals("local")) {
                mConnectionMethod = CvsConnectionMethod.LOCAL;
            } else if (sConnectionMethod.equals("sample")) {
                mConnectionMethod = CvsConnectionMethod.SAMPLE;
            } else if (sConnectionMethod.equals("logfile")) {
                mConnectionMethod = CvsConnectionMethod.LOGFILE;
            } else if (sConnectionMethod.equals("pserver")) {
                mConnectionMethod = CvsConnectionMethod.PSERVER;
            } else if (
                sConnectionMethod.equals("ssh")
                    || sConnectionMethod.equals("ext")
                    || sConnectionMethod.equals("server")) {
                mConnectionMethod = CvsConnectionMethod.EXT;
            } else {
                throw new InvalidRepositoryLocationException(
                    "Unrecognized connection method: " + sConnectionMethod);
            }
        } else {
            if (aLocation.indexOf(':') == -1) {
                mConnectionMethod = CvsConnectionMethod.LOCAL;
            } else {
                throw new InvalidRepositoryLocationException(
                    "Unrecognized connection method: " + aLocation);
            }
        }

        if (!mConnectionMethod.equals(CvsConnectionMethod.LOCAL)) {
            // Check for user and host

            int nEndOfHost = aLocation.indexOf(':', nEndOfConnectionMethod + 1);
            // Host specified
            if (nEndOfHost != -1) {
                String sUserAndHostName =
                    aLocation.substring(nEndOfConnectionMethod + 1, nEndOfHost);
                int nEndOfUser = sUserAndHostName.indexOf('@');
                // User name specified
                if (nEndOfUser != -1) {
                    mUserName = sUserAndHostName.substring(0, nEndOfUser).trim();
                }
                mHostName = sUserAndHostName.substring(nEndOfUser + 1).trim();

                if (mHostName.length() == 0) {
                    throw new InvalidRepositoryLocationException("The host name was not specified");
                }
            } else {
                // host not specified
                throw new InvalidRepositoryLocationException("The host was not specified");
            }

            mRepositoryPath = aLocation.substring(nEndOfHost + 1);
        } else {
            if (aLocation.startsWith(":local:")) {
                mRepositoryPath = aLocation.substring(7);
            } else {
                mRepositoryPath = aLocation;
            }
        }

        if (mRepositoryPath.endsWith("/")) {
            mRepositoryPath = mRepositoryPath.substring(0, mRepositoryPath.length() - 1);
        }

        mRepositoryPath = mRepositoryPath.trim();

        if (mRepositoryPath.length() == 0) {
            throw new InvalidRepositoryLocationException("Repository path not specified");
        }
    }

    /**
     * Creates a RepositoryLocation with asserted valid parameters
     * This method is used internally for local logfile and sample log file access
     * @param aMethod method
     * @param aUserName username
     * @param aHostName hostname
     * @param aRepositoryPath path
     * @return RepositoryLocation
     */
    public static RepositoryLocation createValidLocation(
        CvsConnectionMethod aMethod,
        String aUserName,
        String aHostName,
        String aRepositoryPath) {
        try {
            return new RepositoryLocation(aMethod, aUserName, aHostName, aRepositoryPath);
        } catch (InvalidRepositoryLocationException e) {
            // ignore
        }
        return null;
    }

    /**
     * Gets the connection method for this location
     * @return CvsConnectionMethod
     */
    public CvsConnectionMethod getConnectionMethod() {
        return mConnectionMethod;
    }

    /**
     * Returns the CVSROOT Representation of this Location
     * @return CVSROOT Representation of this Location
     */
    public String getCvsRoot() {
        if (mStringRepresentation == null) {
            StringBuffer buffer = new StringBuffer(30);

            buffer.append(':');
            buffer.append(mConnectionMethod.getCvsRootMethodName());
            buffer.append(':');

            if (!mConnectionMethod.equals(CvsConnectionMethod.LOCAL)) {
                if (mUserName != null) {
                    buffer.append(mUserName);
                    buffer.append('@');
                }

                buffer.append(mHostName);
                buffer.append(':');
            }

            buffer.append(mRepositoryPath);

            mStringRepresentation = buffer.toString();
        }

        return mStringRepresentation;
    }

    /**
     * Gets the host name.
     * @return String
     */
    public String getHostName() {
        return mHostName;
    }

    /**
     * Gets the path to the repository.
     * @return String
     */
    public String getRepositoryPath() {
        return mRepositoryPath;
    }

    /**
     * Gets the user name.
     * @return String
     */
    public String getUserName() {
        return mUserName;
    }
    /**
     * @param aMethod method
     */
    public void setConnectionMethod(CvsConnectionMethod aMethod) {
        mConnectionMethod = aMethod;
    }

    /**
     * Converts this location to a string of the form:<p>
     * <pre>:method:user@host:/path/to/repository</pre>
     * @return String
     */
    public String toString() {
        if (mStringRepresentation == null) {
            StringBuffer buffer = new StringBuffer(30);

            buffer.append(':');
            buffer.append(mConnectionMethod.toString());
            buffer.append(':');

            if (!mConnectionMethod.equals(CvsConnectionMethod.LOCAL)) {
                if (mUserName != null) {
                    buffer.append(mUserName);
                    buffer.append('@');
                }

                buffer.append(mHostName);
                buffer.append(':');
            }

            buffer.append(mRepositoryPath);

            mStringRepresentation = buffer.toString();
        }

        return mStringRepresentation;
    }
    private CvsConnectionMethod mConnectionMethod;
    private String mStringRepresentation;
    private String mUserName, mHostName, mRepositoryPath;

}
