/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import junit.framework.TestCase;
import net.sf.bloof.Bloof;
import net.sf.bloof.db.*;

import java.util.Iterator;
import java.util.Vector;

import ca.utoronto.JCVSReport.cvsPlugin.History;

/** MetricDatabaseTest: tests the ability of the MetricDatabase
 * to correctly store and retrieve CVS history data.
 * @author Polina Gohshtein
 */
public class MetricDatabaseTest extends TestCase {

    private MetricDatabase db = null;

    /** Create a database connection and use it to initialize
     * the Metric Database.
     */
    protected void setUp() throws Exception {
        super.setUp();
        try {
            Connection conn = McKoiControl.create(new DefaultDbAccess(
                    "MyDb", "test", "test",
                    Bloof.LOCAL_BLOOF_DIR + "/testdir"));
            db = new MetricDatabase(conn);
        } catch (Exception e) {
            fail("Failed to connect and initialize MetricDatabase."
                    + e.toString());
        }
    }

    /** Close the database.
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        db.close();
    }

    /** Make sure that MetricDatabase can populate the database
     * correctly with an operation that has a file and a revision
     * associated with it.
     */
    public void testOperationWithRevision() {
        // Create a history iterator
        Vector history = new Vector();
        History record =
            new History("A", Timestamp.valueOf("2005-01-29 03:56:00.0"),
            "g2polgoh", "1.1", "TestFile.java", "408project");
        history.add(record);

        // The database needs to be populated with CVS log data first
        // bacause it the MetricDatabase class relies on the log data to
        // be there first - before the history data gets inserted
        try {
            db.setAutoCommit(false);
            // Insert a developer
            db.executeQuery("INSERT INTO Developer (login) "
                + " VALUES ('g2polgoh')");
            // Insert a file
            db.executeQuery("INSERT INTO File (pathname,name) "
                + "VALUES ('408project/TestFile.java', 'TestFile.java')");
            // Insert a revision
            db.executeQuery("INSERT INTO Revision "
                + "(file,tstamp,developer,version) VALUES "
                + "('408project/TestFile.java', '2005-01-29 03:56:00.0', "
                + "'g2polgoh', '1.1')");
            db.setAutoCommit(true);
        } catch (SQLException e) {
            fail("Failed to insert data into the database. "
                    + e.toString());
        }

        // Get an iterator over the history vector
        Iterator it = history.iterator();
        try {
            db.populateWithHistory(it);
        } catch (SQLException e) {
            fail("Failed to populate database with history record. "
                    + e.toString());
        }

        // Now query the database and compare the results
        try {
            ResultSet rs = db.executeQuery("SELECT * FROM CVSOperation");
            assertTrue(rs.next());
            assertEquals(rs.getString("operation"), "A");
            assertEquals(rs.getString("developer"), "g2polgoh");
            assertEquals(rs.getString("tstamp"), "2005-01-29 03:56:00.0");

        } catch (SQLException e) {
            fail("Failed to insert data into the database. "
                    + e.toString());
        }
    }

    /** Make sure that MetricDatabase can populate the database
     * correctly with an operation that does not have a
     * revision associated with it - such as removing a file.
     */
    public void testOperationWithoutRevision() {
        // Create a history iterator
        Vector history = new Vector();
        History record =
            new History("R", Timestamp.valueOf("2005-01-29 03:56:00.0"),
            "jack", null, "OldFile.java", "408project");
        history.add(record);

        // The database needs to be populated with CVS log data first
        // bacause it the MetricDatabase class relies on the log data to
        // be there first - before the history data gets inserted
        try {
            db.setAutoCommit(false);
            // Insert a developer
            db.executeQuery("INSERT INTO Developer (login) "
                + " VALUES ('jack')");
            // Insert a file
            db.executeQuery("INSERT INTO File (pathname,name) "
                + "VALUES ('408project/OldFile.java', 'OldFile.java')");
            // Insert a revision
            db.executeQuery("INSERT INTO Revision "
                + "(file,tstamp,developer,version) VALUES "
                + "('408project/OldFile.java', '2005-01-29 03:56:00.0', "
                + "'jack', '1.1')");
            // This file will be removes
            db.executeQuery("INSERT INTO DeletedFile (file,tstamp) "
                    + "VALUES ('408project/OldFile.java',"
                    + "'2005-01-29 03:56:00.0')");
            db.setAutoCommit(true);
        } catch (SQLException e) {
            fail("Failed to insert data into the database. "
                    + e.toString());
        }

        // Get an iterator over the history vector
        Iterator it = history.iterator();
        try {
            db.populateWithHistory(it);
        } catch (SQLException e) {
            fail("Failed to populate database with history record. "
                    + e.toString());
        }

        // Now query the database and compare the results
        try {
            ResultSet rs = db.executeQuery("SELECT * FROM CVSOperation");
            assertTrue(rs.next());
            assertEquals(rs.getString("operation"), "R");
            assertEquals(rs.getString("developer"), "jack");
            assertEquals(rs.getString("tstamp"), "2005-01-29 03:56:00.0");

        } catch (SQLException e) {
            fail("Failed to insert data into the database. "
                    + e.toString());
        }
    }
}
