package ca.utoronto.JCVSReport;

import ca.utoronto.JCVSReport.report.*;

import junit.framework.TestCase;

import java.io.*;

import java.util.regex.*;


/**
 * This test class tests metrics displayed by JCVSReport
 *
 * @author Olga Rodimina
 * @author David James
 */
public class PresentationTest extends TestCase {
    String report = null;

    /** Create a report statically so as to avoid having
     * to recreate the report for each and every one of
     * the below tests
     */
    public void setUp() throws Exception {
        // if sample_test_config exists then do nothing,
        // otherwise create new sample_test_config.txt
        File testConfigFile = new File("sample_test_config.txt");

        if (!testConfigFile.exists()) {
            String cwd = System.getProperty("user.dir");
            String cvsroot = cwd + "/test_cvsrepository";

            //  Create file for testing
            String fileContents = "cvsroot=" + cvsroot + "\n"
                + "cvsmodule=project\n"
                + "graphs=CodeSize,CodeComplexity,TestSize,"
                + "DocumentationSize,Conflicts,Operations,NumberOfImports,"
                + "Coupling,CodeSize-PerDeveloper,"
                + "DocumentationSize-PerDeveloper,TestSize-PerDeveloper,"
                + "CodeComplexity-PerDeveloper,Coupling-PerDeveloper,"
                + "NumberOfClasses-PerDeveloper,"
                + "NumberOfMethods-PerDeveloper,"
                + "LinesOfCode-PerDeveloper,NCSS-PerDeveloper,"
                + "CyclomaticComplexity-PerDeveloper,Lines,"
                + "Lines-PerDeveloper";

            FileOutputStream out;
            PrintStream pStream;

            try {
                out = new FileOutputStream("sample_test_config.txt");
                pStream = new PrintStream(out);
                pStream.println(fileContents);
                pStream.close();
            } catch (Exception e) {
                System.err.println("Error while writing to file");
            }
        }

        Config config = new Config();

        // Load config file from disk
        config.load(new FileInputStream("sample_test_config.txt"));

        // Create test project
        JCVSProject testProject = new JCVSProject(config.cvs(),
                config.db(), config.getGraphNames());

        // import project to the database
        testProject.importProject();

        // get graphs that were created for the test project
        ByteArrayOutputStream buf = new ByteArrayOutputStream();
        Report myreport = testProject.getReport();
        myreport.write(new PrintWriter(buf), new File("./test"));
        report = buf.toString();

        // Close the project
        testProject.closeProject();
    }

    /**
     * This method tests whether JCVSReport can build a report
     * without throwing an exception
     *
     * @throws Exception
     */
    public void testBuildReport() throws Exception {
        assertTrue(report != null);
    }

    /**
     * This method tests whether JCVSReport correctly calculates lines of code
     * in the test project
     *
     * @throws Exception
     */
    public void testLinesOfCode() throws Exception {
        assertTrue(-1 != report.indexOf("(Lines of Code, rodimina) = 121"));
        assertTrue(-1 != report.indexOf("(Lines of Code, David) = 0"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates lines of
     * comments in the test project
     *
     * @throws Exception
     */
    public void testLinesOfComments() throws Exception {
        assertTrue(-1 != report.indexOf("(Lines of Comments, rodimina) = 43"));
        assertTrue(-1 != report.indexOf("(Lines of Comments, David) = 0"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates lines of
     * javadocs in the test project
     *
     * @throws Exception
     */
    public void testLinesOfJavaDoc() throws Exception {
        assertTrue(-1 != report.indexOf("(Lines of JavaDoc, rodimina) = 19"));
        assertTrue(-1 != report.indexOf("(Lines of JavaDoc, David) = 0"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates lines with
     * tabs in the test project
     *
     * @throws Exception
     */
    public void testLinesWithTabs() throws Exception {
        assertTrue(-1 != report.indexOf("(Lines With Tabs, rodimina) = 5"));
        assertTrue(-1 != report.indexOf("(Lines With Tabs, David) = 0"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates lines with
     * trailing spaces in the test project
     *
     * @throws Exception
     */
    public void testLinesWithTrailingSpaces() throws Exception {
        Pattern p = Pattern.compile(
                "Lines With Trailing Spaces: \\([\\d:\\w/ ]+, 7\\)");
        assertTrue(p.matcher(report).find());
        assertTrue(-1 != report.indexOf(
                "(Lines With Trailing Spaces, rodimina) = 7"));
        assertTrue(-1 != report.indexOf(
                "(Lines With Trailing Spaces, David) = 0"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates number of
     * methods in the test project
     *
     * @throws Exception
     */
    public void testNumberOfMethods() throws Exception {
        assertTrue(-1 != report.indexOf("(Number of Methods, rodimina) = 9"));
        assertTrue(-1 != report.indexOf("(Number of Methods, David) = 3"));
    }

    /**
     * Test whether NCSS is correctly calculated
     *
     * @throws Exception
     */
    public void testNCSS() throws Exception {
        assertTrue(-1 != report.indexOf(
                "(Noncomment Source Statements, rodimina) = 17"));
        assertTrue(-1 != report.indexOf(
                "(Noncomment Source Statements, David) = 11"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates number of
     * classes in the test project
     *
     * @throws Exception
     */
    public void testNumberOfClasses() throws Exception {
        Pattern p = Pattern.compile("Number of Classes: \\([\\d:\\w/ ]+, 4\\)");
        assertTrue(p.matcher(report).find());
        assertTrue(-1 != report.indexOf("(Number of Classes, rodimina) = 3"));
        assertTrue(-1 != report.indexOf("(Number of Classes, David) = 1"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates number of test
     * classes in the test project
     *
     * @throws Exception
     */
    public void testNumberOfTestClasses() throws Exception {
        Pattern p = Pattern.compile(
                "Number of Test Classes: \\([\\d:\\w/ ]+, 1\\)");
        assertTrue(p.matcher(report).find());
        assertTrue(-1 != report.indexOf(
                "(Number of Test Classes, rodimina) = 0"));
        assertTrue(-1 != report.indexOf("(Number of Test Classes, David) = 1"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates number of
     * tests methods in the test project
     *
     * @throws Exception
     */
    public void testNumberOfTestMethods() throws Exception {
        Pattern p = Pattern.compile(
                "Number of Test Methods: \\([\\d:\\w/ ]+, 2\\)");
        assertTrue(p.matcher(report).find());
        assertTrue(-1 != report.indexOf(
                "(Number of Test Methods, rodimina) = 0"));
        assertTrue(-1 != report.indexOf("(Number of Test Methods, David) = 2"));
    }

    /**
     * This method tests whether JCVSReport correctly calculates cvs operations
     * per developer in the  test project
     *
     * @throws Exception
     */
    public void testOperationsPerDeveloper() throws Exception {
        Pattern p = Pattern.compile("Commits: \\([\\d:\\w/ ]+, 4\\)");
        assertTrue(p.matcher(report).find());
    }

    /**
     * Test the 'Number of Imports' metric
     * @throws Exception
     */
    public void testNumberOfImports() throws Exception {
        assertTrue(-1 != report.indexOf("(Number of Imports, rodimina) = 6"));
        assertTrue(-1 != report.indexOf("(Number of Imports, David) = 3"));
    }

    /**
     * Test the 'Cyclomatic Complexity' metric
     * @throws Exception
     */
    public void testCyclomaticComplexity() throws Exception {
        assertTrue(
            -1 != report.indexOf("(Cyclomatic Complexity, rodimina) = 9")
        );
        assertTrue(-1 != report.indexOf("(Cyclomatic Complexity, David) = 3"));
    }
}
