/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.cvsPlugin;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;

import junit.framework.TestCase;
import net.sf.bloof.scm.ScmRevisionIterator;
import net.sf.bloof.scm.cvsplugin.CvsAccess;
import net.sf.bloof.scm.cvsplugin.Revision;
import ca.utoronto.JCVSReport.Config;

/** CvsTest contains unit tests that verify that the CVS class
 * gets the right data as output by running the cvs log and history
 * commands. It also verifies that it can get the contents of a
 * specified version of a file.
 * @author Polina Gohshtein
 */
public class CvsTest extends TestCase {

    CvsAccess cvsAccess = null;

    /**
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {

        // if sample_test_config exists then do nothing,
        // otherwise create new sample_test_config.txt
        File testConfigFile = new File("sample_test_config.txt");

        if (!testConfigFile.exists()) {
            String cwd = System.getProperty("user.dir");
            String cvsroot = cwd + "/test/test_cvsrepository";

            //  Create file for testing
            String fileContents = "cvsroot="
                    + cvsroot
                    + "\n"
                    + "cvsmodule=project\n"
                    + "graphs=CodeSize,CodeComplexity,TestSize,"
                    + "DocumentationSize,Conflicts,"
                    + "Operations,NumberOfImports,Coupling,"
                    + "CodeSize-PerDeveloper,DocumentationSize-PerDeveloper,"
                    + "TestSize-PerDeveloper,CodeComplexity-PerDeveloper,"
                    + "Coupling-PerDeveloper,"
                    + "NumberOfClasses-PerDeveloper,"
                    + "NumberOfMethods-PerDeveloper,"
                    + "LinesOfCode-PerDeveloper,NCSS-PerDeveloper,"
                    + "CyclomaticComplexity-PerDeveloper,Lines,"
                    + "Lines-PerDeveloper";

            FileOutputStream out;
            PrintStream pStream;

            try {
                out = new FileOutputStream("sample_test_config.txt");
                pStream = new PrintStream(out);
                pStream.println(fileContents);
                pStream.close();
            } catch (Exception e) {
                System.err.println("Error while writing to file");
            }
        }

        // Load config file from disk
        Config config = new Config();
        config.load(new FileInputStream("sample_test_config.txt"));
        cvsAccess = config.cvs();
    }

    /**
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        cvsAccess = null;
    }

    /** testGetAllCvsHistory: tests the ability of the CVS class
     * to get the cvs history data from the cvs repository.
     */
    public void testGetAllCvsHistory() {

        assertNotNull(cvsAccess);
        Cvs cvs = new Cvs(cvsAccess);

        try {
            // Get all cvs history data
            Iterator history = cvs.getCvsHistoryData();

            /* Count the number of history records.
             * There should be 9 records.
             *
                Operation: M; time: 2005-03-13 02:27:00.0; user: David; path
                Operation: M; time: 2005-03-12 19:12:00.0; user: rodimina;
                Operation: M; time: 2005-03-12 19:12:00.0; user: rodimina;
                Operation: M; time: 2005-03-12 19:12:00.0; user: rodimina;
                Operation: A; time: 2005-03-12 19:13:00.0; user: rodimina;
                Operation: A; time: 2005-03-12 19:13:00.0; user: rodimina;
                Operation: A; time: 2005-03-12 19:13:00.0; user: rodimina;
                Operation: M; time: 2005-03-12 19:35:00.0; user: rodimina;
                Operation: A; time: 2005-03-12 19:35:00.0; user: rodimina;
             *
             */

            int nRecords = 0;
            while (history.hasNext()) {
                History record = (History) history.next();
                nRecords++;
            }
            assertEquals(nRecords, 9);
        } catch (Exception e) {
            fail("Failed to get cvs history data from test repository."
                    + e.toString());
        }
    }

    /** testGetCvsLogFromDate: tests the ability of the CVS class
     * to get the cvs log data from a specific date.
     */
    public void testGetCvsLogFromDate() {

        assertNotNull(cvsAccess);
        Cvs cvs = new Cvs(cvsAccess);

        try {
            // Get all cvs log data from a specific date until now
            DateFormat formatter = new SimpleDateFormat("MM/dd/yy");
            Date date = (Date) formatter.parse("03/13/05");
            ScmRevisionIterator logs = cvs.getCvsLogData(date);

            /* Count the number of revisions.
             * There should be no revisions. */

            int nRevisions = 0;
            while (logs.hasNext()) {
                Revision revision = (Revision) logs.getNext();
                nRevisions++;
            }
            assertEquals(nRevisions, 0);
        } catch (Exception e) {
            fail("Failed to get cvs log data from test repository."
                    + e.toString());
        }
    }

    /** testGetAllCvsLogData: : tests the ability of the CVS class
     * to get the cvs log data from the cvs repository.
     */
    public void testGetAllCvsLogData() {

        assertNotNull(cvsAccess);
        Cvs cvs = new Cvs(cvsAccess);

        try {
            // Get all cvs log data
            ScmRevisionIterator logs = cvs.getCvsLogData();

            /* Count the number of revisions.
             * There should be 25 revisions.

                File: ProjectFile1.java; Rev: 1.2
                File: ProjectFile1.java; Rev: 1.1
                File: ProjectFile1.java; Rev: 1.1.1.1
                File: ProjectFile2.java; Rev: 1.3
                File: ProjectFile2.java; Rev: 1.2
                File: ProjectFile2.java; Rev: 1.1
                File: ProjectFile2.java; Rev: 1.1.1.1
                File: TestFile1.java; Rev: 1.3
                File: TestFile1.java; Rev: 1.2
                File: TestFile1.java; Rev: 1.1
                File: TestFile1.java; Rev: 1.1.1.1
                File: file.c; Rev: 1.1
                File: file.c; Rev: 1.1.1.1
                File: file.py; Rev: 1.1
                File: file.py; Rev: 1.1.1.1
                File: ProjectFile1_rev1.txt; Rev: 1.1
                File: ProjectFile1_rev1.txt; Rev: 1.1.1.1
                File: ProjectFile1_rev2.txt; Rev: 1.1
                File: ProjectFile2_rev1.txt; Rev: 1.1
                File: ProjectFile2_rev1.txt; Rev: 1.1.1.1
                File: ProjectFile2_rev2.txt; Rev: 1.1
                File: ProjectFile2_rev3.java; Rev: 1.1
                File: TestFile1_rev1.txt; Rev: 1.1
                File: TestFile1_rev1.txt; Rev: 1.1.1.1
                File: TestFile1_rev2.txt; Rev: 1.1

             *
             */

            int nRevisions = 0;
            while (logs.hasNext()) {
                Revision revision = (Revision) logs.getNext();
                nRevisions++;
            }
           assertEquals(nRevisions, 25);
        } catch (Exception e) {
            fail("Failed to get cvs log data from test repository."
                    + e.toString());
        }
    }

    /** testGetCvsHistoryFromDate: : tests the ability of the CVS class
     * to get the cvs history data from a specific date.
     */
    public void testGetCvsHistoryFromDate() {

        assertNotNull(cvsAccess);
        Cvs cvs = new Cvs(cvsAccess);

        try {
            // Get all cvs history data
            DateFormat formatter = new SimpleDateFormat("MM/dd/yy");
            Date date = (Date) formatter.parse("03/13/05");

            Iterator history = cvs.getCvsHistoryData(date);
            /* Count the number of history records.
             * There should be no history records. */

            int nRecords = 0;
            while (history.hasNext()) {
                History record = (History) history.next();
                nRecords++;
            }
            assertEquals(nRecords, 0);
        } catch (Exception e) {
            fail("Failed to get cvs history data from test repository."
                    + e.toString());
        }
    }

    /** testGetCvsFile: tests the ability of the CVS class to get
     * the contents of a given revision of a file.
     */
    /*
     public void testGetCvsFile() {

        assertNotNull(cvsAccess);
        Cvs cvs = new Cvs(cvsAccess);

        try {

            String file = cvsAccess.getCvsRoot()
                + "/project/src/ProjectFile1.java";
            String version = "1.2";

            String contents = cvs.getCvsFile(file, version);
            System.out.println(contents);

        } catch (Exception e) {
            fail("Failed to get a file from the test repository."
                    + e.toString());
        }
    }
    */
}
