/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.metric;

import junit.framework.TestCase;

import java.util.*;
import java.util.regex.*;


/**
 * Test case to test RegexLineCounter
 *
 * @author Olga Rodimina Test Case for RegexLineCounterTest
 */
public class RegexLineCounterTest extends TestCase {
    Pattern regex; // Regular expression to match
    String testJavaFile, testJavaAST; // file contents
    RegexLineCounter lineCounter; // RegexLineCounter that will be testted
    String propertyLocation = "ca.utoronto.JCVSReport.metric.properties.";

    /**
     * setUp method creates RegexLineCounter object that will be used to
     * calculate desired statistics
     *
     * @throws Exception
     */
    protected void setUp() throws Exception {
        lineCounter = new RegexLineCounter();

        // Initialize test string
        testJavaFile = "import net.sf.bloof.util.intl.Language;\n"
            + "import net.sf.bloof.util.intl.Messages;\n"
            + "import net.sf.bloof.util.intl.Text;\n"
            + "\t\t\t\n"
            + "\t\t\n"
            + "/**\n" + "*\n"
            + "  * @version $Id: LanguageTest.java\n"
            + "  */\n"
            + "  public class LanguageTest extends TestCase {\n"
            + "\t\t\t\n"
            + "\t\t//comment\t\n"
            + "\t\t\n"
            + "        /**\n"
            + "          * Tests the Internationalization of Bloof \n"
            + "          */\n"
            + "             public void testLanguages() {             \n"
            + "             String langId;\n"
            + "             langId  =  Messages.getString();\n"
            + "             assertEquals( ENGLISH, langId );\n"
            + "             Messages.setLanguage( Language.GERMAN );\n";

        testJavaAST =
            "PackageDeclaration                          \n"
          + " Name:net.sf.bloof                          \n"
          + "TypeDeclaration                             \n"
          + " InterfaceDeclaration:(public)              \n"
          + "  UnmodifiedInterfaceDeclaration            \n"
          + "   InterfaceMemberDeclaration               \n"
          + "    MethodDeclaration:(package private)     \n"
          + "     ResultType                             \n"
          + "     MethodDeclarator(bloofExceptionHappend)\n"
          + "      FormalParameters                      \n"
          + "       FormalParameter:(package private)    \n"
          + "        Type                                \n"
          + "         Name:BloofException                \n";

    }

    /** Count the number of lines with method declarators */
    public void testNumMethodDeclarators() {
        regex = Pattern.compile("MethodDeclarator.*\n");
        int numMatches = lineCounter.countMatches(testJavaAST, regex);
        assertEquals(1, numMatches);
    }

    /**
     * Tests whether RegexLineCounter calculates correctly number of lines of
     * code in 'testContents' string
     */
    public void testNumMatchesLinesOfCode() {
        ResourceBundle props = ResourceBundle.getBundle(
                propertyLocation + "LinesOfCode");

        // get regular expression for the metric
        regex = Pattern.compile(props.getString("regex"));
        int numMatches = lineCounter.countMatches(testJavaFile, regex);
        assertEquals(21, numMatches);
    }

    /**
     * Tests whether RegexLineCounter calculates correctly number of lines of
     * comments in 'testContenets' string
     */
    public void testNumMatchesLinesOfComments() {
        ResourceBundle props = ResourceBundle.getBundle(
                propertyLocation + "LinesOfComments");

        // get regular expression for the metric
        regex = Pattern.compile(props.getString("regex"));
        int numMatches = lineCounter.countMatches(testJavaFile, regex);
        assertEquals(8, numMatches);
    }

    /**
     * Tests whether RegexLineCounter calculates correctly number of javadocs
     * lines in 'testContenets' string
     */
    public void testNumMatchesLinesOfJavaDocs() {
        ResourceBundle props = ResourceBundle.getBundle(
                propertyLocation + "LinesOfJavaDoc");

        // get regular expression for the metric
        regex = Pattern.compile(props.getString("regex"));
        int numMatches = lineCounter.countMatches(testJavaFile, regex);
        assertEquals(7, numMatches);
    }

    /**
     * Tests whether RegexLineCounter calculates correctly number of lines with
     * tabs in 'testContenets' string
     */
    public void testNumMatchesLinesWithTabs() {
        ResourceBundle props = ResourceBundle.getBundle(
                propertyLocation + "LinesWithTabs");

        // get regular expression for the metric
        regex = Pattern.compile(props.getString("regex"));
        int numMatches = lineCounter.countMatches(testJavaFile, regex);
        assertEquals(5, numMatches);
    }

    /**
     * Tests whether RegexLineCounter calculates correctly number of lines with
     * trailing spaces in 'testContenets' string
     */
    public void testNumMatchesLinesWithTrailingSpaces() {
        ResourceBundle props = ResourceBundle.getBundle(
                propertyLocation + "LinesWithTrailingSpaces");

        // get regular expression for the metric
        regex = Pattern.compile(props.getString("regex"));
        int numMatches = lineCounter.countMatches(testJavaFile, regex);
        assertEquals(7, numMatches);
    }


    /**
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }
}
