/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.report;

import ca.utoronto.JCVSReport.metric.MetricRevision;

import junit.framework.TestCase;

import java.sql.*;

import java.util.ResourceBundle;

/**
 * Test case for BasicGraph
 *
 * @author Olga Rodimina
 */
public class BasicGraphTest extends TestCase {
    MockGraph testGraph;
    MetricRevision revision;

    /**
     * Sets up TestGraph object to test BasicGraph
     *
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        testGraph = new MockGraph();

        // initialize MetricRevision that will be used for testing
        revision = new MetricRevision("LinesOfCode", "file1",
                new Timestamp(3251253), 5.0, "rodimina");

        // Initialize 'testGraph' to be associated with the
        // TestGraph.properties file
        ResourceBundle props = ResourceBundle.getBundle(
                "ca.utoronto.JCVSReport.report.TestGraph");

        // intialize test graph
        testGraph.init(null, "TestGraph", props);
    }

    /**
     * Tests if the title of the graph was set correctly from  properties file
     * associated with this graph
     */
    public void testGetTitle() {
        String title = testGraph.getTitle();
        assertEquals(title, "Lines of code over time");
    }

    /**
     * Tests if the height of the graph was set correctly from properties file
     * associated with this graph
     */
    public void testGetHeight() {
        int height = testGraph.getHeight();
        assertEquals(height, 200);
    }

    /**
     * Tests if the width of the graph was set correctly from properties file
     * associated with this graph
     */
    public void testGetWidth() {
        int width = testGraph.getWidth();
        assertEquals(width, 600);
    }

    /**
     * Tests if the XLabel of the graph was set correctly from properties file
     * associated with this graph
     */
    public void testGetXLabel() {
        assertEquals(testGraph.getXLabel(), "Date");
    }

    /**
     * Tests if the YLabel of the graph was set correctly from properties file
     * associated with this graph
     */
    public void testGetYLabel() {
        assertEquals(testGraph.getYLabel(), "Lines");
    }

    /**
     * Tests whether correct key is returned for the testGraph
     */
    public void testGetKey() {
        assertEquals("LinesOfCode", testGraph.getKey(revision));
    }

    /**
     * Tests whether the type of graph is set correctly by the
     * 'setPerDeveloper' method
     */
    public void testSetPerDeveloperTrue() {
        testGraph.setPerDeveloper(true);
        assertTrue(testGraph.perDeveloper);
    }

    /**
     * Tests whether the type of graph is set correctly by the
     * 'setPerDeveloper' method
     */
    public void testSetPerDeveloperFalse() {
        testGraph.setPerDeveloper(false);
        assertFalse(testGraph.perDeveloper);
    }

    /**
     * Tests if the correct seriesLabel is returned for the specified  revision
     */
    public void testGetSeriesLabel() {
        assertEquals("Lines of Code", testGraph.getSeriesLabel(revision));
    }

    /**
     * Tests if the join() correctly joins array of string specified to it
     */
    public void testJoin() {
        String token = ",";
        String[] strings = {"Str1", "Str2", "Str3"};
        String joinResult = testGraph.join(token, strings);
        assertEquals("Str1,Str2,Str3", joinResult);
    }

    /**
     * Tests if the metric value is stored correctly in the graph for the
     * specified revision
     */
    public void testStoreMetricValue() {
        testGraph.storeMetricValue(revision);
        assertTrue(testGraph.devMap.containsKey(revision));
        assertTrue(testGraph.devMap.containsKey(revision));
    }

    /**
     * Test wheter metric value is stored correctly in the graph for the
     * specified revision
     */
    public void testGetStoredMetricValue() {
        double value = testGraph.getStoredMetricValue(revision);
        assertEquals(0, (int) value);
    }

    /**
     * Tests wether correct metric names are returned for the test graph
     */
    public void testGetMetricNames() {
        String[] names = testGraph.getMetricNames();
        String expectedNames = "LinesOfCode,LinesOfComments,LinesOfJavaDoc,"
            + "LinesWithTabs,LinesWithTrailingSpaces";
        assertEquals(expectedNames, testGraph.join(",", names));
    }

    /**
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }
}
