/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.JCVSReport.report;

import ca.utoronto.JCVSReport.metric.MetricRevision;

import junit.framework.TestCase;

import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;

import java.sql.*;

import java.util.Date;

/**
 * Test class for TimeSeriesGraph
 *
 * @author Olga Rodimina
 */
public class TimeSeriesGraphTest extends TestCase {
    // revision used for the test methods
    MetricRevision revision;

    // test graph
    TimeSeriesGraph graph;

    // test series representing data points of the test graph
    TimeSeries testSeries;

    // time stamp for testing
    Timestamp timestamp;

    /**
     * Set up testing environment for the TimeSeriesGraph
     *
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        Date now = new Date();
        long nowLong = now.getTime();

        timestamp = new Timestamp(nowLong);

        // initialize revision for testing
        revision = new MetricRevision("LinesOfCode", "file1", timestamp, 5.0,
                "rodimina");

        // create time series object for testing
        testSeries = new TimeSeries("LinesOfCode");
        testSeries.addOrUpdate(new Day(01, 01, 2005), 1);
        testSeries.addOrUpdate(new Day(02, 01, 2005), 2);
        testSeries.addOrUpdate(new Day(03, 01, 2005), 3);

        // instantiate graph
        graph = new TimeSeriesGraph();
    }

    /**
     * Tests whether the revsion is added to the current series
     *
     * @throws Exception
     */
    public void testAddRevisionToSeriesNewItem() throws Exception {
        graph.addRevisionToSeries(testSeries, revision);
        assertEquals(revision.getDay(), testSeries.getTimePeriod(3));
        assertEquals(4, testSeries.getItemCount());
        assertEquals((Number) new Double(5.0), testSeries.getValue(3));
    }

    /**
     * Test whether addRevisionToSeries inserts duplicate items
     *
     * @throws Exception
     */
    public void testAddRevisionToSeriesExistingItem() throws Exception {
        graph.addRevisionToSeries(testSeries, revision);
        assertEquals(revision.getDay(), testSeries.getTimePeriod(3));
        assertEquals(4, testSeries.getItemCount());
        assertEquals((Number) new Double(5.0), testSeries.getValue(3));
    }

    /**
     * Tests whether timeSeriesGraph fills correctly interim days
     *
     * @throws Exception
     */
    public void testFillInterimDays() throws Exception {
        Date now = new Date();
        long nowLong = now.getTime();

        timestamp = new Timestamp(nowLong);

        // initialize revision for testing
        revision = new MetricRevision("LinesOfCode", "file1", timestamp, 6.0,
                "rodimina");

        graph.fillInterimDays(testSeries, revision);
        assertEquals((Number) new Double(1.0), testSeries.getValue(0));
        assertEquals((Number) new Double(2.0), testSeries.getValue(1));
        assertEquals((Number) new Double(3.0), testSeries.getValue(2));
        assertEquals(3, testSeries.getItemCount());
    }

    /**
     * Tests whether timeSeriesGraph fills correctly interim days when the time
     * period to fill is specified
     *
     * @throws Exception
     */
    public void testInterimDaysFromTo() throws Exception {
        graph.fillInterimDays(testSeries, testSeries.getTimePeriod(0),
            testSeries.getTimePeriod(2), 4.0);
        assertEquals((Number) new Double(1.0), testSeries.getValue(0));
        assertEquals((Number) new Double(4.0), testSeries.getValue(1));
        assertEquals((Number) new Double(3.0), testSeries.getValue(2));
    }
}
