/*
 *  JCVSReport - Assess the Progress of a Java Project in CVS
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.utoronto.SimpleProcess;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;

import ca.utoronto.JCVSReport.Config;
import junit.framework.TestCase;
import java.io.IOException;

/**
 * Test the SimpleProcess class
 */
public class SimpleProcessTest extends TestCase {

    // Example commands
    String[] command1;
    String[] command2 = {"cvs"};
    String[] command3 = {"non-existent"};

    /** Set up our tests
     */
    protected void setUp() throws Exception {

        // if sample_test_config exists then do nothing,
        // otherwise create new sample_test_config.txt
        File testConfigFile = new File("sample_test_config.txt");

        if (!testConfigFile.exists()) {
            String cwd = System.getProperty("user.dir");
            String cvsroot = cwd + "/test_cvsrepository";

            //  Create file for testing
            String fileContents = "cvsroot="
                    + cvsroot
                    + "\n"
                    + "cvsmodule=project\n"
                    + "graphs=CodeSize,CodeComplexity,TestSize,"
                    + "DocumentationSize,Conflicts,"
                    + "Operations,NumberOfImports,Coupling,"
                    + "CodeSize-PerDeveloper,DocumentationSize-PerDeveloper,"
                    + "TestSize-PerDeveloper,CodeComplexity-PerDeveloper,"
                    + "Coupling-PerDeveloper,"
                    + "NumberOfClasses-PerDeveloper,"
                    + "NumberOfMethods-PerDeveloper,"
                    + "LinesOfCode-PerDeveloper,NCSS-PerDeveloper,"
                    + "CyclomaticComplexity-PerDeveloper,Lines,"
                    + "Lines-PerDeveloper";

            FileOutputStream out;
            PrintStream pStream;

            try {
                out = new FileOutputStream("sample_test_config.txt");
                pStream = new PrintStream(out);
                pStream.println(fileContents);
                pStream.close();
            } catch (Exception e) {
                System.err.println("Error while writing to file");
            }
        }

        // Load config file from disk
        Config config = new Config();
        config.load(new FileInputStream("sample_test_config.txt"));
        command1 = new String[] {"cvs", "-d", config.cvs().getCvsRoot(),
            "version"};
    }

    /** Test the run method */
    public void testSimpleProcess() throws Exception {
        // Test with null command
        boolean threwError = false;
        try {
            new SimpleProcess(null);
        } catch (NullPointerException ex) {
            threwError = true;
        }
        assertTrue(threwError);

        // The remainder of the tests are tested by the testWaitFor
        // subroutine, so we'll use that
        testWaitFor();
    }


    /** Test the run method */
    public void testRun() throws Exception {

        // Test with working command
        String output = SimpleProcess.run(command1);
        assertTrue(-1 != output.indexOf("Concurrent Versions System"));

        // Test with non-zero status
        boolean threwError = false;
        try {
            output = SimpleProcess.run(command2);
        } catch (InterruptedException ex) {
            threwError = true;
        }
        assertTrue(threwError);

        // Test with non-existent command
        threwError = false;
        try {
            output = SimpleProcess.run(command3);
        } catch (IOException ex) {
            threwError = true;
        }
        assertTrue(threwError);

        // Test with null command
        threwError = false;
        try {
            output = SimpleProcess.run(null);
        } catch (NullPointerException ex) {
            threwError = true;
        }
        assertTrue(threwError);
    }


    /** Test the stderr method */
    public void testStderr() throws Exception {

        // Test with no output
        SimpleProcess process = new SimpleProcess(command1);
        new StreamGobbler(process.stdout()).start();
        assertEquals(null, process.stderr().readLine());
        new StreamGobbler(process.stderr()).start();
        assertEquals(0, process.waitFor());

        // Test with output
        process = new SimpleProcess(command2);
        new StreamGobbler(process.stdout()).start();
        assertTrue(-1 != process.stderr().readLine().indexOf("Usage"));
        new StreamGobbler(process.stderr()).start();
        assertEquals(1, process.waitFor());

    }

    /** Test the stdout method */
    public void testStdout() throws Exception {

        // Test with no output
        SimpleProcess process = new SimpleProcess(command1);
        new StreamGobbler(process.stdout()).start();
        assertEquals(null, process.stderr().readLine());
        new StreamGobbler(process.stderr()).start();
        assertEquals(0, process.waitFor());

        // Test with output
        process = new SimpleProcess(command2);
        new StreamGobbler(process.stdout()).start();
        assertTrue(-1 != process.stderr().readLine().indexOf("Usage"));
        new StreamGobbler(process.stderr()).start();
        assertEquals(1, process.waitFor());

    }

    /** Test the waitFor method */
    public void testWaitFor() throws Exception {

        // Test with command that returns success status
        SimpleProcess process = new SimpleProcess(command1);
        new StreamGobbler(process.stdout()).start();
        new StreamGobbler(process.stderr()).start();
        assertEquals(0, process.waitFor());

        // Test with command that returns error status
        process = new SimpleProcess(command2);
        new StreamGobbler(process.stdout()).start();
        new StreamGobbler(process.stderr()).start();
        assertEquals(1, process.waitFor());

        // Test with non-existent command
        boolean threwError = false;
        try {
            process = new SimpleProcess(command3);
            new StreamGobbler(process.stdout()).start();
            new StreamGobbler(process.stderr()).start();
            assertEquals(1, process.waitFor());
        } catch (IOException ex) {
            threwError = true;
        }
        assertTrue(threwError);
    }

}
